{****************************************************}
{ 		CToolPens.p}
{}
{		The ToolPens Class}
{}
{		SUPERCLASS = CPaintTask}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CToolPens;

interface

	uses
		TCL, MoreTCL, ArtClassIntf;

	const
		SHIFTKEY_CODE = $38;	{ Hardware code for the shift key	}
		iPAINTING = 1;			{ Index for Painting task name		}
		iERASER = 2;				{ Index for Erase task name			}
		iPENCIL = 3;				{ Index for Pencil task name			}

		ERASER_LENGTH = 16;	{ Size of eraser in pixels			}
		BRUSH_LENGTH = 4;		{ Size of brush in pixels				}

implementation



{*** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ***}



{****************************************************}
{ BeginTracking (OVERRIDE)}
{}
{		Mouse down tracking is about to begin at the specified starting}
{		point. Set the pen size and fill pattern.}
{}
{****************************************************}

	procedure CToolPens.BeginTracking (var startPt: Point);
	begin
						{ Copy entire image to scratch pad	}

		CopyBits(itsPainting.macBitMap, gScratchPad.macBitMap, itsPainting.macBitMap.bounds, itsPainting.macBitMap.bounds, srcCopy, nil);

		PenNormal;
	end;


{****************************************************}
{ KeepTracking (OVERRIDE)}
{}
{		Continuous mouse tracking while the mouse button is down. Draw}
{		only when the mouse moves or the panorama autoscrolls.}
{}
{****************************************************}

	procedure CToolPens.KeepTracking (var currPt, prevPt, startPt: Point);
		var
			currRect: Rect;

	begin
		if itsPaintPane.AutoScroll(currPt) or not EqualPt(currPt, prevPt) then
			begin
				itsPaintPane.GetFrame(currRect);
				PinInRect(currRect, currPt);
				UseTool(currPt, prevPt);
			end;
	end;


{****************************************************}
{ CalcPenRect}
{}
{		Calculate the rectangle affected by a pen tool located at a}
{		specified point}
{}
{****************************************************}

	procedure CToolPens.CalcPenRect (thePt: Point; var penRect: Rect);
	begin
		penRect.left := thePt.h;				{ Pen hangs below and to the right	}
		penRect.top := thePt.v;				{   of its coordinate location			}
		penRect.right := thePt.h + penLength;
		penRect.bottom := thePt.v + penLength;
	end;


{****************************************************}
{ CalcDirtyRect}
{}
{		Calculate the portion of the image drawn on by a pen. Find the}
{		rectangle which encloses the current dirtyRect and the current}
{		pen rectangle.}
{}
{****************************************************}

	procedure CToolPens.CalcDirtyRect (penRect: Rect);
	begin
		dirtyRect.left := Min(dirtyRect.left, penRect.left);
		dirtyRect.top := Min(dirtyRect.top, penRect.top);
		dirtyRect.right := Max(dirtyRect.right, penRect.right);
		dirtyRect.bottom := Max(dirtyRect.bottom, penRect.bottom);
	end;


{****************************************************}
{ CalcDrawRgn}
{}
{		Calculate the draw region affected by a pen. What a pain. We have}
{		to tweak the region depending on where the current point is in}
{		relation to the previous point because the pen hangs below and}
{		to the right of its location.}
{}
{****************************************************}

	procedure CToolPens.CalcDrawRgn (currRect, prevRect: Rect; drawRgn: RgnHandle);
	begin
		OpenRgn;

		if currRect.left > prevRect.left then
			begin
				if currRect.top > prevRect.top then
					begin
						MoveTo(currRect.right, currRect.top);
						LineTo(currRect.right, currRect.bottom);
						LineTo(currRect.left, currRect.bottom);
						LineTo(prevRect.left, prevRect.bottom);
						LineTo(prevRect.left, prevRect.top);
						LineTo(prevRect.right, prevRect.top);
						LineTo(currRect.right, currRect.top);

					end
				else
					begin
						MoveTo(currRect.left, currRect.top);
						LineTo(currRect.right, currRect.top);
						LineTo(currRect.right, currRect.bottom);
						LineTo(prevRect.right, prevRect.bottom);
						LineTo(prevRect.left, prevRect.bottom);
						LineTo(prevRect.left, prevRect.top);
						LineTo(currRect.left, currRect.top);
					end;

			end
		else
			begin
				if currRect.top > prevRect.top then
					begin
						MoveTo(currRect.right, currRect.bottom);
						LineTo(currRect.left, currRect.bottom);
						LineTo(currRect.left, currRect.top);
						LineTo(prevRect.left, prevRect.top);
						LineTo(prevRect.right, prevRect.top);
						LineTo(prevRect.right, prevRect.bottom);
						LineTo(currRect.right, currRect.bottom);

					end
				else
					begin
						MoveTo(currRect.left, currRect.bottom);
						LineTo(currRect.left, currRect.top);
						LineTo(currRect.right, currRect.top);
						LineTo(prevRect.right, prevRect.top);
						LineTo(prevRect.right, prevRect.bottom);
						LineTo(prevRect.left, prevRect.bottom);
						LineTo(currRect.left, currRect.bottom);
					end;
			end;

		CloseRgn(drawRgn);
	end;


{****************************************************}
{ EndTracking (OVERRIDE)}
{}
{		Mouse down tracking has ended because the user has released the}
{		mouse button. Its now time to set up the undo buffer and change}
{		the actual offscreen painting image.}
{}
{****************************************************}

	procedure CToolPens.EndTracking (var currPt, prevPt, startPt: Point);
        var
            theUndoBits: CBitMap;       { Altered by TCL Weaver 1.0 (5/9/90) }

	begin
									{ Track for the last time				}

		KeepTracking(currPt, prevPt, startPt);

									{ Create an undo bitmap				}

        new(theUndoBits);           { Altered by TCL Weaver 1.0 (5/9/90) }
        itsUndoBits := theUndoBits;
		gApplication.RequestMemory(TRUE, TRUE);
		itsUndoBits.IBitMap(dirtyRect.right - dirtyRect.left, dirtyRect.bottom - dirtyRect.top, FALSE);
		gApplication.RequestMemory(FALSE, FALSE);

		if itsUndoBits.macBitMap.baseAddr = nil then
			begin
				itsUndoBits.Free;
				itsUndoBits := nil;

			end
		else
			begin
				itsUndoBits.SetBoundsOrigin(dirtyRect.left, dirtyRect.top);

									{ Original image is on the scratchpad	}
									{ Copy dirty area to undo bitmap		}

				CopyBits(gScratchPad.macBitMap, itsUndoBits.macBitMap, dirtyRect, dirtyRect, srcCopy, nil);
			end;
	end;


{****************************************************}
{ UseTool}
{}
{****************************************************}

	procedure CToolPens.UseTool (var currPt, prevPt: Point);
	begin
	end;									{ Null Method							}



{*** Methods for Different Pen Tools ***}



{****************************************************}
{ IToolEraser (Eraser)}
{}
{****************************************************}

	procedure CToolEraser.IToolEraser (aPaintPane: CPaintPane; aPainting: CBitMap);
	begin
		IPaintTask(iERASER, aPaintPane, aPainting);

		penLength := ERASER_LENGTH;
	end;


{****************************************************}
{ BeginTracking (Eraser)}
{}
{****************************************************}

	procedure CToolEraser.BeginTracking (var startPt: Point);
		var
			r: Rect;

	begin
		inherited BeginTracking(startPt);

		CalcPenRect(startPt, r);
		dirtyRect := r;

		EraseRect(r);						{ Do erasure on screen				}

		itsPainting.BeginDrawing;		{ Do erasure on offscreen bitmap	}
		EraseRect(r);
		itsPainting.EndDrawing;
	end;


{****************************************************}
{ UseTool (Eraser)}
{}
{****************************************************}

	procedure CToolEraser.UseTool (var currPt, prevPt: Point);
		var
			currRect, prevRect: Rect;

	begin
		CalcPenRect(currPt, currRect);
		CalcPenRect(prevPt, prevRect);

		CalcDirtyRect(currRect);			{ Determine changed area			}
		CalcDrawRgn(currRect, prevRect, gUtilRgn);

		EraseRgn(gUtilRgn);					{ Do erasure on screen				}

		itsPainting.BeginDrawing;			{ Do erasure on offscreen bitmap	}
		EraseRgn(gUtilRgn);
		itsPainting.EndDrawing;
	end;


{****************************************************}
{ IToolPencil (Pencil)}
{}
{****************************************************}

	procedure CToolPencil.IToolPencil (aPaintPane: CPaintPane; aPainting: CBitMap);
	begin
		IPaintTask(iPENCIL, aPaintPane, aPainting);

		penLength := 1;
	end;


{****************************************************}
{ BeginTracking (Pencil)}
{}
{****************************************************}

	procedure CToolPencil.BeginTracking (var startPt: Point);
		var
			r: Rect;
			thePenPat: Pattern;

	begin
		inherited BeginTracking(startPt);

		CalcPenRect(startPt, r);
		dirtyRect := r;

		PenNormal;						{ Start with a normal pen			}

		if itsPainting.PixelIsBlack(startPt) then
										{ Pencil is on a black pixel			}
										{   Subsequent drawing will create	}
										{   white pixels						}
			BlockMove(@white, @thePenPat, 8)

		else 								{ Pencil is on a white pixel			}
										{   Subsequent drawing will create	}
										{   black pixels						}
			BlockMove(@black, @thePenPat, 8);

		PenPat(thePenPat);					{ Do drawing on screen			}
		MoveTo(startPt.h, startPt.v);
		Line(0, 0);

		itsPainting.BeginDrawing;		{ Do drawing on offscreen bitmap	}
		PenPat(thePenPat);
		MoveTo(startPt.h, startPt.v);
		Line(0, 0);
		itsPainting.EndDrawing;
	end;


{****************************************************}
{ UseTool (Pencil)}
{}
{****************************************************}

	procedure CToolPencil.UseTool (var currPt, prevPt: Point);
		var
			currRect: Rect;

	begin
		CalcPenRect(currPt, currRect);	{ Determine changed area			}
		CalcDirtyRect(currRect);

		MoveTo(prevPt.h, prevPt.v);		{ Do drawing on screen				}
		LineTo(currPt.h, currPt.v);

		itsPainting.BeginDrawing;			{ Do drawing on offscreen bitmap	}
		MoveTo(prevPt.h, prevPt.v);
		LineTo(currPt.h, currPt.v);
		itsPainting.EndDrawing;
	end;


{****************************************************}
{ IToolBrush (Paint Brush)}
{}
{****************************************************}

	procedure CToolBrush.IToolBrush (aPaintPane: CPaintPane; aPainting: CBitMap);
	begin
		IPaintTask(iPAINTING, aPaintPane, aPainting);

		penLength := BRUSH_LENGTH;
	end;


{****************************************************}
{ BeginTracking (Paint Brush)}
{}
{****************************************************}

	procedure CToolBrush.BeginTracking (var startPt: Point);
		var
			r: Rect;

	begin
		inherited BeginTracking(startPt);

		CalcPenRect(startPt, r);
		dirtyRect := r;

		FillRect(r, gPattern);			{ Do painting on screen				}

		itsPainting.BeginDrawing;		{ Do painting on offscreen bitmap	}
		FillRect(r, gPattern);
		itsPainting.EndDrawing;
	end;


{****************************************************}
{ UseTool (Paint Brush)}
{}
{****************************************************}

	procedure CToolBrush.UseTool (var currPt, prevPt: Point);
		var
			currRect, prevRect: Rect;

	begin
		CalcPenRect(currPt, currRect);
		CalcPenRect(prevPt, prevRect);

		CalcDirtyRect(currRect);						{ Determine changed area	}
		CalcDrawRgn(currRect, prevRect, gUtilRgn);

		FillRgn(gUtilRgn, gPattern);		{ Do painting on screen				}

		itsPainting.BeginDrawing;		{ Do painting on offscreen bitmap	}
		FillRgn(gUtilRgn, gPattern);
		itsPainting.EndDrawing;
	end;


end.