{****************************************************}
{}
{ 		CBartender.p																		}
{}
{		The Bartender Class																}
{}
{		Manages menus and the menu bar. Maintains a table which maps menu		}
{		choices into command numbers.													}
{}
{		SUPERCLASS = CObject															}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CBartender;

interface

	uses
		TCL;


implementation



{    * * * * C O N S T R U C T I O N / D E S T R U C T I O N      M E T H O D S * * * *  }



{****************************************************}
{ IBartender}
{}
{		Initialize a Bartender object}
{}
{****************************************************}

	procedure CBartender.IBartender (MBARid: integer);
		var
			theMenuIDs: IntArrayH;	{ List of MENU id numbers 				}
			MBARlength: integer;		{ Number of menus in MBAR resource 	}
			i: integer;					{ Index into number of menus 			}
			menuID: integer;			{ MENU resource ID 						}
			theEntry: MenuEntry;		{ Entry in the menu table 				}

	begin

{	Menus to install at startup are specified in a MBAR resource. }
{	MBAR resource is just an array of MENU ids (integers) where }
{	the first integer is a count of the number of menus. }

		theMenuIDs := IntArrayH(GetResource('MBAR', MBARid));
		CheckResource(Handle(theMenuIDs));
		MBARlength := theMenuIDs^^[0];

{	Allocate a menu table with an entry for each menu }

		numMenus := MBARlength;
		theMenus := MenuArrayH(NewHandleClear(numMenus * sizeof(MenuEntry)));

{	It's important to use a local variable to store the number of }
{	menus in the MBAR since the numMenus instance variable }
{	may change within the loop because of hierarchical menus }

		for i := 0 to MBARlength - 1 do
			begin
				menuID := theMenuIDs^^[i + 1];
				theEntry.MENUid := menuID;
				theEntry.macMenu := MenuHandle(GetMenu(menuID));
				CheckResource(Handle(theEntry.macMenu));
				theEntry.dimming := DEFAULT_DIM;
				theEntry.unchecking := FALSE;
				theEntry.hasHMenus := FALSE;

{	We append command numbers to the text of item names. }

				ExtractCommands(theEntry);
				theMenus^^[i] := theEntry;

{	Automatically install any hier menus. }

				ExtractHierMenus(theEntry.macMenu, i);

{	Build up the menu bar from left to right. }

				InsertMenu(theEntry.macMenu, 0);
			end;

		ReleaseResource(Handle(theMenuIDs));
		DrawMenuBar;
	end;



{****   I N S E R T I O N / D E L E T I O N   M E T H O D S   ****}



{****************************************************}
{	 AddMenu}
{}
{	        Add a menu to the menu table, if it's not already in it, and optionally}
{	        install the menu in the menu bar.}
{}
{****************************************************}

	procedure CBartender.AddMenu (MENUid: integer; install: Boolean; beforeID: integer);
		var
			i: integer;					{ Index into menu table 	}
			theEntry: MenuEntry;		{ Entry in the menu table	}

	begin
		i := FindMenuIndex(MENUid);		{ Check if menu is in our table }

		if i < 0 then		{ Menu not in our table -- add it }
			begin
				i := numMenus;
				numMenus := numMenus + 1;

				SetHandleSize(Handle(theMenus), numMenus * sizeof(MenuEntry));

				theEntry.MENUid := MENUid;
				theEntry.macMenu := GetMenu(MENUid);
				CheckResource(Handle(theEntry.macMenu));
				theEntry.dimming := DEFAULT_DIM;
				theEntry.unchecking := FALSE;
				theEntry.hasHMenus := FALSE;
				ExtractCommands(theEntry);
				theMenus^^[i] := theEntry;
				ExtractHierMenus(theEntry.macMenu, i);
			end;

		if install then
			begin
				InsertMenu(theMenus^^[i].macMenu, beforeID);
				if beforeID <> hierMenu then
					begin
						DeleteFromBar(NOTHING);		{ Force redraw of menu bar }
					end;
			end;
	end;


{****************************************************}
{	 RemoveMenu}
{}
{	        Remove a menu from the menu bar and the menu table. The associated}
{	        Toolbox menu record is also disposed.}
{}
{****************************************************}

	procedure CBartender.RemoveMenu (MENUid: integer);
		var
			i: integer;			{ Index into menu table }

	begin
		DeleteFromBar(MENUid);		{ Remove from menu bar }
		i := FindMenuIndex(MENUid);

		if i >= 0 then
			begin
				ReleaseResource(Handle(theMenus^^[i].macMenu));
				numMenus := numMenus - 1;
				if i < numMenus then
					BlockMove(@theMenus^^[i + 1], @theMenus^^[i], longint((numMenus - i) * sizeof(MenuEntry)));
				SetHandleSize(Handle(theMenus), numMenus * sizeof(MenuEntry));
			end;

{ +++ else debugging error msg (Menu not in menu table) }
	end;


{****************************************************}
{	 InsertInBar}
{}
{	        Place a menu in the menu bar. Same as the AddMenu method with}
{	        install set to TRUE.}
{}
{****************************************************}

	procedure CBartender.InsertInBar (MENUid, beforeID: integer);
	begin
		AddMenu(MENUid, TRUE, beforeID);
	end;


{****************************************************}
{	 DeleteFromBar}
{}
{	        Remove a menu from the menu bar, but leave it in the menu table.}
{}
{****************************************************}

	procedure CBartender.DeleteFromBar (MENUid: integer);
	begin
		if MENUid <> NOTHING then
			DeleteMenu(MENUid);			{ Remove menu from menu bar }
		HiliteMenu(NOTHING);
		DrawMenuBar;
	end;


{****************************************************}
{ InsertHierMenu}
{}
{		Attach a hierarchical menu within an existing menu. The command}
{		number for the hier menu is needed if you want to be able to}
{		dim/highlight the entire hier menu. The title of the hier menu}
{		is used as the text for the menu item within its parent menu.}
{}
{****************************************************}

	procedure CBartender.InsertHierMenu (hMENUid: integer; cmdNo: longint; inMENUid, afterItem: integer);
		var
			macMenu: MenuHandle;
			i, inMenuNumCmds: integer;

	begin
		i := FindMenuIndex(inMENUid);

		if i >= 0 then
			begin
				AddMenu(hMENUid, TRUE, hierMenu);
				macMenu := GetMHandle(hMENUid);

				inMenuNumCmds := theMenus^^[i].numCmds;

				if afterItem < 0 then			{ afterItem must be between 0 and	}
					afterItem := 0					{   the total number of items	 in the	}
											{	target menu.						}

				else if afterItem > inMenuNumCmds then
					afterItem := inMenuNumCmds;

				InsertMenuCmd(cmdNo, macMenu^^.menuData, inMENUid, afterItem);

				macMenu := GetMHandle(inMENUid);
				SetItemMark(macMenu, afterItem + 1, chr(hMENUid));
				SetItemCmd(macMenu, afterItem + 1, chr(hMenuCmd));

				theMenus^^[i].hasHMenus := TRUE;
			end;
	end;



{****   I T E M   M A N I P U L A T I O N   M E T H O D S    ****}



{****************************************************}
{ EnableCmd}
{}
{		Enable the menu item corresponding to the specified command number}
{}
{****************************************************}

	procedure CBartender.EnableCmd (cmdNo: longint);
		var
			MENUid: integer;
			itemNo: integer;
			macMenu: MenuHandle;

	begin
		FindMenuItem(cmdNo, MENUid, macMenu, itemNo);
		if macMenu <> nil then
			EnableItem(macMenu, itemNo);
	end;


{****************************************************}
{ DisableCmd}
{}
{		Disable the menu item corresponding to the specified command number}
{}
{****************************************************}

	procedure CBartender.DisableCmd (cmdNo: longint);
		var
			MENUid: integer;
			itemNo: integer;
			macMenu: MenuHandle;

	begin
		FindMenuItem(cmdNo, MENUid, macMenu, itemNo);
		if macMenu <> nil then
			DisableItem(macMenu, itemNo);
	end;


{****************************************************}
{	 EnableMenu}
{}
{	        Enable a menu as a whole. Note that items disabled separately, such}
{	        as by using the DisableCmd method, will remain disabled. Uninstalled}
{	        menus cannot be enabled.}
{}
{****************************************************}

	procedure CBartender.EnableMenu (MENUid: integer);
		var
			macMenu: MenuHandle;

	begin
		macMenu := GetMHandle(MENUid);		{ Get associated Toolbox record }
		if macMenu <> nil then
			begin
				EnableItem(macMenu, 0);			{ Enable the menu }
				DeleteFromBar(NOTHING);			{ Force redraw of menu bar }
			end;
	end;


{****************************************************}
{	 DisableMenu}
{}
{	        Disable a menu as a whole. The menu title and all items in the}
{	        menu will be dimmed. Uninstalled menus cannot be disabled.}
{}
{****************************************************}

	procedure CBartender.DisableMenu (MENUid: integer);
		var
			macMenu: MenuHandle;

	begin
		macMenu := GetMHandle(MENUid);		{ Get associated Toolbox record }
		if macMenu <> nil then
			begin
				DisableItem(macMenu, 0);			{ Disable the menu }
				DeleteFromBar(NOTHING);			{ Force redraw of menu bar }
			end;
	end;


{****************************************************}
{	 SetCmdText}
{}
{	        Specify the text for a menu command}
{}
{****************************************************}

	procedure CBartender.SetCmdText (cmdNo: longint; theText: Str255);
		var
			MENUid: integer;
			macMenu: MenuHandle;
			itemNo: integer;

	begin
		FindMenuItem(cmdNo, MENUid, macMenu, itemNo);
		if macMenu <> nil then
			SetItem(macMenu, itemNo, theText);
	end;


{****************************************************}
{	 GetCmdText}
{}
{	        Retrieve the text for a menu command}
{}
{****************************************************}

	procedure CBartender.GetCmdText (cmdNo: longint; var theText: Str255);
		var
			MENUid: integer;
			macMenu: MenuHandle;
			itemNo: integer;

	begin
		FindMenuItem(cmdNo, MENUid, macMenu, itemNo);
		if macMenu <> nil then
			GetItem(macMenu, itemNo, theText);
	end;


{****************************************************}
{	 CheckMarkCmd}
{}
{	        Place or remove a check mark before the menu item corresponding}
{	        to a specified command. If checked is TRUE, a check mark is}
{	        placed; if checked is FALSE, the check mark is removed.}
{}
{****************************************************}

	procedure CBartender.CheckMarkCmd (cmdNo: longint; checked: Boolean);
		var
			MENUid: integer;
			macMenu: MenuHandle;
			itemNo: integer;

	begin
		FindMenuItem(cmdNo, MENUid, macMenu, itemNo);
		if MENUid <> NOTHING then
			CheckItem(macMenu, itemNo, checked);
	end;



{*** I T E M   I N S E R T I O N / D E L E T I O N   M E T H O D S ***}



{****************************************************}
{ InsertMenuCmd}
{}
{		Insert a menu command in a menu at a position following a specified}
{		item}
{}
{****************************************************}

	procedure CBartender.InsertMenuCmd (cmdNo: longint; theText: Str255; MENUid, afterItem: integer);
		var
			i: integer;						{ Index into menu table			}
			oldNumCmds: integer;
			theCommandsP: LongArrayP;

	begin
		i := FindMenuIndex(MENUid);		{ Locate Menu in the menu table	}

		if i >= 0 then						{ If Menu is in the menu table ...	}
			begin
				oldNumCmds := theMenus^^[i].numCmds;

				if afterItem < 0 then			{ afterItem must be between 0 and	}
					afterItem := 0					{   the total number of items			}

				else if afterItem > oldNumCmds then
					afterItem := oldNumCmds;


										{ Add item to Mac menu				}

				InsMenuItem(theMenus^^[i].macMenu, theText, afterItem);

										{ Now add new command to our list	}

				theMenus^^[i].numCmds := oldNumCmds + 1;			{ Make room for a new item in list	}

				if oldNumCmds = 0 then
					theMenus^^[i].theCommands := LongArrayH(NewHandle(4))
				else
					SetHandleSize(Handle(theMenus^^[i].theCommands), (oldNumCmds + 1) * 4);

				theCommandsP := theMenus^^[i].theCommands^;

										{ Shift items down one slot		}

				if afterItem < oldNumCmds then
					BlockMove(@theCommandsP^[afterItem], @theCommandsP^[afterItem + 1], (oldNumCmds - afterItem) * 4);


				theCommandsP^[afterItem] := cmdNo;	{ Store command number. Note that	}
													{   location [afterItem] is really		}
													{   the afterItem+1 item in the 		}
													{   list (difference between zero	}
													{   and one-based lists)				}
			end;
	end;


{****************************************************}
{ RemoveMenuCmd}
{}
{		Remove the specified command from the Bartender.}
{}
{****************************************************}

	procedure CBartender.RemoveMenuCmd (cmdNo: longint);
		var
			MENUid: integer;
			macMenu: MenuHandle;
			itemNo: integer;
			i, oldNumCmds: integer;

	begin
					{ Look up command in menu table	}

		FindMenuItem(cmdNo, MENUid, macMenu, itemNo);

		if MENUid <> NOTHING then			{ If command is in our table ...	}
			begin
				i := FindMenuIndex(MENUid);

				DelMenuItem(macMenu, itemNo);	{ Remove item from Mac menu		}

				oldNumCmds := theMenus^^[i].numCmds;

					{ Shift items up one slot			}

				if itemNo < oldNumCmds then
					BlockMove(@theMenus^^[i].theCommands^^[itemNo], @theMenus^^[i].theCommands^^[itemNo - 1], (oldNumCmds - itemNo) * 4);


				theMenus^^[i].numCmds := oldNumCmds - 1;
				SetHandleSize(Handle(theMenus^^[i].theCommands), (oldNumCmds - 1) * 4);
			end;
	end;



{*** L O O K - U P   M E T H O D S ***}



{****************************************************}
{	 FindMenuIndex}
{}
{	        Find the index number of a menu in the menu table}
{}
{****************************************************}

	function CBartender.FindMenuIndex (MENUid: integer): integer;
		var
			i: integer;
			theMenusP: MenuArrayP;

	begin
		FindMenuIndex := -1;
		theMenusP := theMenus^;
		for i := 0 to numMenus - 1 do
			if theMenusP^[i].MENUid = MENUid then
				begin
					FindMenuIndex := i;
					Leave;
				end;
	end;


{****************************************************}
{ FindMacMenu}
{}
{		Find the Mac MenuHandle associated with a specified MENU ID. Return}
{		nil if ID is not in our menu table.}
{}
{****************************************************}

	function CBartender.FindMacMenu (MENUid: integer): MenuHandle;
		var
			index: integer;			{ Index in menu table				}

	begin
		index := FindMenuIndex(MENUid);		{ Search menu table for this ID	}

								{ Non-negative index means ID is	}
								{   in our table						}

		if index >= 0 then
			FindMacMenu := theMenus^^[index].macMenu
		else
			FindMacMenu := nil;
	end;


{****************************************************}
{ FindCmdNumber}
{}
{		Find the command number corresponding to the specified Menu choice,}
{		which is indicated by a MENU resource ID and item number.}
{}
{****************************************************}

	function CBartender.FindCmdNumber (MENUid, itemNo: integer): longint;
		var
			i: integer;
			cmdNo: longint;

	begin
		i := FindMenuIndex(MENUid);

		if (i >= 0) and (itemNo > 0) then
			begin
				if itemNo <= theMenus^^[i].numCmds then
					begin
						cmdNo := theMenus^^[i].theCommands^^[itemNo - 1];
						if cmdNo >= 0 then
							begin
								FindCmdNumber := cmdNo;
								Exit(FindCmdNumber);
							end;
					end;
				FindCmdNumber := (-(BSL(longint(MENUid), 16)) - itemNo);
			end

		else
			FindCmdNumber := cmdNull;
	end;


{****************************************************}
{ FindMenuItem}
{}
{		Find the associated menu ID, menu handle, and item number for a specified}
{		command number}
{}
{****************************************************}

	procedure CBartender.FindMenuItem (cmdNo: longint; var MENUid: integer; var macMenu: MenuHandle; var itemNo: integer);
		var
			i, j: integer;
			cmdNoTry: longint;
			theMenusP: MenuArrayP;

	begin
		if cmdNo < 0 then
			begin
				MENUid := HiWord(-cmdNo);
				itemNo := LoWord(-cmdNo);
				macMenu := MenuHandle(GetResource('MENU', MENUid));
			end

		else
			begin
				theMenusP := theMenus^;
				for i := 0 to numMenus - 1 do
					for j := 0 to theMenusP^[i].numCmds - 1 do
						begin
							cmdNoTry := theMenusP^[i].theCommands^^[j];
							if cmdNo = cmdNoTry then
								begin
									MENUid := theMenusP^[i].MENUid;
									macMenu := theMenusP^[i].macMenu;
									itemNo := j + 1;
									Exit(FindMenuItem);
								end;
						end;
				MENUid := NOTHING;
				macMenu := nil;
				itemNo := NOTHING;
			end;
	end;


{****************************************************}
{ FindItemText}
{}
{		Find the item number in a Menu which has the specified text. A}
{		value of zero (0) is returned if there is no such item or menu.}
{}
{****************************************************}

	function CBartender.FindItemText (MENUid: integer; itemStr: Str255): integer;
		var
			macMenu: MenuHandle;
			numItems: integer;
			i: integer;
			testStr: Str255;

	begin
		macMenu := FindMacMenu(MENUid);	{ Get Mac menu record given ID	}

		if macMenu <> nil then 				{ Make sure menu exists	}
			begin
							{ Run thru all items in the menu }

				numItems := CountMItems(macMenu);
				for i := 1 to numItems do
					begin
						GetItem(macMenu, i, testStr);

							{ Comparison is case-sensitive and	}
							{   diacritical marks matter		}

						if EqualString(itemStr, testStr, TRUE, TRUE) then
							begin
								FindItemText := i;		{ Match found. Exit here			}
								Exit(FindItemText);
							end;
					end;
			end;
								{ Fell out of loop		}
		FindItemText := 0;		{   No match found	}
	end;



{**** C O M M A N D   E X T R A C T I O N ****}



{****************************************************}
{	 ExtractCommands}
{}
{	        Command numbers are appended to the strings for names of}
{	        menu items. We need to separate the item name and the command}
{	        number and store the spearate entities in the proper places.}
{}
{****************************************************}

	procedure CBartender.ExtractCommands (var theEntry: MenuEntry);
		var
			itemNo: integer;		{ Item number in the menu }
			itemStr: Str255;		{ Text of menu item }
			cmdList: LongArrayP;	{ List of menu commands }
			cmdNo: longint;			{ A particular command number }

	begin
		theEntry.numCmds := CountMItems(theEntry.macMenu);
		if theEntry.numCmds = 0 then
			theEntry.theCommands := nil
		else
			begin
				theEntry.theCommands := LongArrayH(NewHandle(theEntry.numCmds * 4));
				HLock(Handle(theEntry.theCommands));
				cmdList := theEntry.theCommands^;
				for itemNo := 1 to theEntry.numCmds do
					begin
						GetItem(theEntry.macMenu, itemNo, itemStr);
						ParseItemString(itemStr, cmdNo);
						cmdList^[itemNo - 1] := cmdNo;
						SetItem(theEntry.macMenu, itemNo, itemStr);
					end;
				HUnlock(Handle(theEntry.theCommands));
			end;
	end;


{****************************************************}
{	 ParseItemString}
{}
{	        Chop the command number off the end of the item string. Return}
{	        both the shortened string and the command number.}
{}
{****************************************************}

	procedure CBartender.ParseItemString (var itemStr: Str255; var cmdNo: longint);
		var
			len: integer;		{ Length of the item string }
			cmdStr: Str255;	{ Holds characters of command no. }
			i: integer;			{ Index into item string }
			j: integer;			{ Index into command no. string }

	begin
		len := length(itemStr);
        cmdStr := nullStr;      { Altered by TCL Weaver 1.0 (4/27/90) }
		for i := 1 to len do
			if itemStr[i] = CMD_DELIMITER then
				begin
					for j := i + 1 to len do
						begin
							cmdStr[j - i] := itemStr[j];
							itemStr[j] := chr(0);
						end;
					itemStr[i] := chr(0);
{$PUSH}
{$R-}
					itemStr[0] := chr(i - 1);
					cmdStr[0] := chr(len - i);
{$POP}
					Leave;
				end;
		StringToNum(cmdStr, cmdNo);
	end;


{****************************************************}
{	 ExtractHierMenus}
{}
{	        Check a Toolbox menu record for hierarchical menus and automatically}
{	        add them to the Toolbox menu list.}
{}
{****************************************************}

	procedure CBartender.ExtractHierMenus (macMenu: MenuHandle; index: integer);
		var
			i: integer;
			hMENUid, itemCmd: char;

	begin

{	Run backwards through items in the menu.  Backwards because we }
{	don't want to put the call to CountMItems as the exit condition, }
{	which is evaluated each time through the loop.  The order of }
{	insertion of hierarchical menus is not significant. }

		for i := CountMItems(macMenu) downto 1 do
			begin
				GetItemCmd(macMenu, i, itemCmd);
				if ord(itemCmd) = hMenuCmd then
					begin
						theMenus^^[index].hasHMenus := TRUE;
						GetItemMark(macMenu, i, hMENUid);
						if ord(hMENUid) <> NOTHING then
							AddMenu(ord(hMENUid), TRUE, hierMenu);
					end;
			end;
	end;



{*** I T E M   D I M M I N G / E N A B L I N G   M E T H O D S ***}



{****************************************************}
{ SetDimOption}
{}
{		Specify the DimOption for a Menu. Items in all menus are dimmed}
{		according to this option whenever the menus are updated.}
{}
{		Choices are:}
{}
{			dimNONE - Don't dim any items in the menu}
{			dimSOME - Dim items with defined command numbers}
{						  Items 1 thru numCmds}
{			dimALL  - Dim all items}
{}
{****************************************************}

	procedure CBartender.SetDimOption (MENUid: integer; aDimming: DimOption);
		var
			index: integer;

	begin
		index := FindMenuIndex(MENUid);		{ Find menu in our table			}

		if index >= 0 then						{ Set option if menu is in table	}
			theMenus^^[index].dimming := aDimming;
	end;


{****************************************************}
{ SetUncheckOption}
{}
{		Specify the UncheckOption for a Menu. Specifying TRUE means that}
{		all items in the menu are automatically unchecked whenever menus}
{		are updated.}
{}
{****************************************************}

	procedure CBartender.SetUnchecking (MENUid: integer; anUnchecking: Boolean);
		var
			index: integer;

	begin
		index := FindMenuIndex(MENUid);		{ Find menu in our table			}

		if index >= 0 then						{ Set option if menu is in table	}
			theMenus^^[index].unchecking := anUnchecking;
	end;


{****************************************************}
{ UpdateAllMenus}
{}
{		First dim all menu items, then let everyone in the active chain}
{		of command update their menu items}
{}
{****************************************************}

	procedure CBartender.UpdateAllMenus;
		var
			i, shift: integer;
			mask: longint;
			j, numItems: integer;
			hasHMenus: Boolean;
			macMenu: MenuHandle;
			itemCmd: char;
			theMenusP: MenuArrayP;

	begin
		HLock(Handle(theMenus));
		theMenusP := theMenus^;

		for i := 0 to numMenus - 1 do		{ Run thru all menus				}
			begin
				macMenu := theMenusP^[i].macMenu;

						{ Dim items according to the		}
						{   specific option for the menu	}

				if theMenusP^[i].dimming = dimALL then
					macMenu^^.enableFlags := BAND(macMenu^^.enableFlags, 1)

				else if theMenusP^[i].dimming = dimSOME then
					begin

						{ Dim items 1 thru numCmds	}

						shift := theMenusP^[i].numCmds + 1;
						if shift > 1 then
							begin						{ numCmds is 0 if shift is 1			}
								if shift > 31 then
									mask := 1				{ Only 31 items can be dimmed		}
								else
									mask := BSL($FFFFFFFF, shift) + 1;

								macMenu^^.enableFlags := BAND(macMenu^^.enableFlags, mask);
							end;
					end;

				if theMenusP^[i].unchecking then
					begin
						numItems := CountMItems(macMenu);
						if theMenusP^[i].hasHMenus then
							for j := 1 to numItems do
								begin
									GetItemCmd(macMenu, j, itemCmd);
									if itemCmd <> chr(hMenuCmd) then
										CheckItem(macMenu, j, FALSE);
								end

						else
							for j := 1 to numItems do
								CheckItem(macMenu, j, FALSE);
					end;
			end;

		HUnlock(Handle(theMenus));

		gGopher.UpdateMenus;			{ Let active Bureaucrats update	}
										{   the menu items				}
	end;

end.