{****************************************************}
{ 		CClipboard.p}
{}
{		The Clipboard Class}
{}
{	    	Implements the "clipboard" used for copying and pasting information }
{	    	which is supported by almost all Mac applications. This class uses }
{	    	the global scrap and can store and retrieve PICT and TEXT data. Only }
{	    	one item may be stored in the Clipboard at a time. This class also }
{	    	supports a window which displays the information on the Clipboard. }
{}
{	    	Hooks are provided for subclasses which can use a private scrap and }
{	    	store any application-specific information. }
{}
{		SUPERCLASS = CDirector}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CClipboard;

interface

	uses
		TCL;


implementation



{ **** C O N S T R U C T I O N / D E S T R U C T I O N     M E T H O D S **** }



{****************************************************}
{ IClipboard}
{}
{		Initialize a Clipboard object}
{}
{****************************************************}

	procedure CClipboard.IClipboard (aSupervisor: CApplication; hasWindow: Boolean);

    { Altered by TCL Weaver 1.0 (5/9/90) }

        var
            theWindow: CWindow;
            theScrollPane: CScrollPane;

	begin
		IDirector(aSupervisor);		{ Initialize superclass			}

		if hasWindow then
			begin

        { Altered by TCL Weaver 1.0 (5/9/90) }

                new(theWindow);
                new(theScrollPane);

                itsWindow := theWindow;
                itsScrollPane := theScrollPane;

                itsWindow.IWindow(WIND_CLIPBOARD, FALSE, gDesktop, SELF);

				itsScrollPane.IScrollPane(itsWindow, SELF, 0, 0, 0, 0, sizELASTIC, sizELASTIC, TRUE, TRUE, TRUE);
				itsScrollPane.FitToEnclFrame(TRUE, TRUE);
			end

		else
			itsScrollPane := nil;

		itsContents := nil;
		theLength := 0;
		theOffset := 0;
		lastScrapCount := 0;
		privateNewer := FALSE;
		windowVisible := FALSE;
	end;


{****************************************************}
{ Suspend (OVERRIDE)}
{}
{		Application is about to be swapped out under MultiFinder or a DA}
{  	    has become active. Hide the clipboard window to avoid displaying}
{  	    incorrect information, since the clipboard contents may change while}
{  	    we are away.}
{}
{****************************************************}

	procedure CClipboard.Suspend;
	begin
		if Status = PRIVATE_SCRAP_NEWER then
			begin
				ConvertPrivate;
				ScrapConverted;
			end;

		if windowVisible then
			begin
				itsWindow.HideSuspend;
				if active then
					begin
						itsWindow.Deactivate;
						active := TRUE;
					end;
			end;
	end;


{****************************************************}
{ Resume (OVERRIDE)}
{}
{		Application is about to be swapped in under MultiFinder or after}
{		a DA was running}
{}
{****************************************************}

	procedure CClipboard.Resume;
	begin
		if (Status = GLOBAL_SCRAP_NEWER) and windowVisible then
			begin
				ConvertGlobal;
				ScrapConverted;
				UpdateDisplay;
			end;

		if windowVisible then
			begin
				itsWindow.ShowResume;
				if active then
					begin
						itsWindow.Activate;
						gGopher := SELF;
					end;
			end;
	end;


{****************************************************}
{ Close}
{}
{ 	    Close Clipboard by closing the clipboard window. Clipboard}
{ 	    doesn't care whether the application is quitting or not.}
{}
{****************************************************}

	function CClipboard.Close (quitting: Boolean): Boolean;
	begin
		if windowVisible then
			begin
				if gInBackground then
					itsWindow.ShowResume;
				CloseWind(itsWindow);
			end;
		Close := TRUE;
	end;


{****************************************************}
{ CloseWind (OVERRIDE)}
{}
{		The clipboard window needs to be closed, usually as a result of}
{		the user clicking in its close box or choosing the "Close" menu}
{		item. All we have to do is hide the window.}
{}
{****************************************************}

	procedure CClipboard.CloseWind (theWindow: CWindow);
		var
			clipStatus: Str255;

	begin
		theWindow.Hide;
		windowVisible := FALSE;
		Deactivate;

		GetIndString(clipStatus, STRcommon, strSHOWCLIP);
		gBartender.SetCmdText(cmdToggleClip, clipStatus);
	end;


{****************************************************}
{ Toggle}
{}
{		Toggle the visibility of the clipboard. If it is visible, hide it;}
{		if it is invisible, show it.}
{}
{****************************************************}

	procedure CClipboard.Toggle;
		var
			clipStatus: Str255;

	begin
		if itsWindow <> nil then
			if windowVisible then
				CloseWind(itsWindow)
			else
				begin
					if Status = GLOBAL_SCRAP_NEWER then
						begin
							ConvertGlobal;
							ScrapConverted;
						end;
					itsWindow.Select;
					windowVisible := TRUE;
					UpdateDisplay;

					GetIndString(clipStatus, STRcommon, strHIDECLIP);
					gBartender.SetCmdText(cmdToggleClip, clipStatus);
				end;
	end;


{****************************************************}
{ PutGlobalScrap}
{}
{		Place data on the global desk scrap}
{}
{****************************************************}

	procedure CClipboard.PutGlobalScrap (theType: ResType; theData: Handle);
		var
			saveHState: SignedByte;	{ Original handle attributes		}
			errCode: longint;			{ Mac error code					}
			temp: Boolean;

	begin
		temp := gError.CheckOSError(ZeroScrap);
		saveHState := HGetState(theData);				{ Save state of data handle		}
		HLock(theData);									{ Lock data in place				}

		errCode := PutScrap(GetHandleSize(theData), theType, theData^);

		HSetState(theData, saveHState);				{ Restore state of data handle	}

		if errCode <> noErr then
			temp := gError.CheckOSError(errCode);
	end;


{****************************************************}
{ GetGlobalScrap}
{}
{		Retrieve data from the global desk scrap. Return}
{		FALSE if no data is found.}
{}
{****************************************************}

	function CClipboard.GetGlobalScrap (theType: ResType; theData: Handle): Boolean;
		var
			length: longint;		{ Amount of data or error code	}
			offset: longint;		{ Offset from start of scrap		}
			temp: Boolean;

	begin
		length := GetScrap(theData, theType, offset);		{ Read from the desk scrap	}

		theLength := length;		{ Save in case someone wants this	}
		theOffset := offset;		{   information						}

		if length < 0 then
			begin					{ Check for an error					}
								{ Finding no data of the specified	}
								{   type does not warrant an error	}
								{   message							}

				if length <> noTypeErr then
					temp := gError.CheckOSError(length);
				GetGlobalScrap := FALSE;					{ No data found					}
			end

		else
			GetGlobalScrap := TRUE;					{ Data successfully retrieved		}
	end;


{****************************************************}
{ Status}
{}
{		Return the status of the clipboard by determining whether the}
{		global or private scrap is more recent}
{}
{****************************************************}

	function CClipboard.Status: ScrapStatus;
		var
			theScrapInfo: PScrapStuff;

	begin
		if privateNewer then	{ Private scrap has changed		}

			Status := PRIVATE_SCRAP_NEWER

		else
			begin
				theScrapInfo := InfoScrap;		{ Get info about global scrap		}

				if theScrapInfo^.scrapCount > lastScrapCount then
					Status := GLOBAL_SCRAP_NEWER
				else
					Status := SCRAPS_THE_SAME
			end;
	end;


{****************************************************}
{ ScrapConverted}
{}
{		Set flags after the scrap has been converted}
{}
{****************************************************}

	procedure CClipboard.ScrapConverted;
		var
			theScrapInfo: PScrapStuff;

	begin
		theScrapInfo := InfoScrap;						{ Get info about global scrap		}
		lastScrapCount := theScrapInfo^.scrapCount;
		privateNewer := FALSE;
	end;


{****************************************************}
{ ConvertGlobal}
{}
{		Take data from the global desk scrap and put it in our private}
{		scrap, performing any necessary conversions. Subclasses which}
{		support a private scrap must override this method.}
{}
{****************************************************}

	procedure CClipboard.ConvertGlobal;
	begin
	end;										{ Null Method						}


{****************************************************}
{ ConvertPrivate}
{}
{		Place information from our private scrap into the global desk scrap,}
{		performing any necessary conversions. Standard types for the}
{		global desk scrap are 'PICT' and 'TEXT'. Subclasses which support}
{		a private scrap must override this method.}
{}
{****************************************************}

	procedure CClipboard.ConvertPrivate;
	begin
	end;										{ Null Method						}


{****************************************************}
{ PutData}
{}
{		Place information on our scrap. This method uses the global desk}
{		scrap. Subclasses must override this method to implement a}
{		private scrap.}
{}
{		NOTE: Subclasses overriding this method should invoke the}
{				PrivateChanged method after placing new data in the}
{				private scrap.}
{}
{****************************************************}

	procedure CClipboard.PutData (theType: ResType; theData: Handle);
	begin
		PutGlobalScrap(theType, theData);
		PrivateChanged;
	end;


{****************************************************}
{ GetData}
{}
{		Retrieve information from our scrap. This method uses the global desk}
{		scrap. If successful, function returns TRUE and passes back a}
{		newly created handle to the data retrieved. Function returns FALSE}
{		if no data was retrieved. Subclasses must override this method to}
{		implement a private scrap.}
{}
{****************************************************}

	function CClipboard.GetData (theType: ResType; var theData: Handle): Boolean;
	begin
		theData := NewHandle(0);						{ Create space for the data		}

		if GetGlobalScrap(theType, theData) then		{ Data successfully retrieved	}
			GetData := TRUE
		else
			begin											{ Some kind of error occurred	}
				DisposHandle(theData);
				theData := nil;								{ Kill the data handle				}
				GetData := FALSE;							{ Signal failure					}
			end;
	end;


{****************************************************}
{ DataSize}
{}
{		Retrieve the length of the data of the specified type which is in}
{		the clipboard. Zero (0) is returned if there is no such data. This}
{		method uses the global desk scrap. Subclasses must override this}
{		method to implement a private scrap.}
{}
{****************************************************}

	function CClipboard.DataSize (theType: ResType): longint;
	begin
		if GetGlobalScrap(theType, nil) then
			DataSize := theLength
		else
			DataSize := 0;
	end;


{****************************************************}
{ PrivateChanged}
{}
{		Private scrap has changed. Set flag and update contents of the}
{		clipboard window, if visible.}
{}
{****************************************************}

	procedure CClipboard.PrivateChanged;
	begin
		privateNewer := TRUE;
		UpdateDisplay;
	end;


{****************************************************}
{ UpdateDisplay}
{}
{		Read from the clipboard and set up a pane to display the contents.}
{		Try to find a 'PICT' first, then 'TEXT'. Give up if neither of}
{		these resource types is present.}
{}
{****************************************************}

	procedure CClipboard.UpdateDisplay;
		var
			theData: Handle;
			margin: Rect;
            theContents: CPanorama;     { Altered by TCL Weaver 1.0 (5/9/90) }

	begin
		if windowVisible then
			begin
				if itsContents <> nil then
					begin
						itsContents.Refresh;
						itsContents.Free;
						itsContents := nil;
						itsScrollPane.itsPanorama := nil;
					end;

				if GetData('PICT', theData) then
					begin
                        new(CPicture(theContents)); { Altered by TCL Weaver 1.0 (5/9/90) }
                        itsContents := theContents;
						CPicture(itsContents).IPicture(itsScrollPane, SELF, 1, 1, 0, 0, sizELASTIC, sizELASTIC);
						CPicture(itsContents).SetMacPicture(PicHandle(theData));
						itsContents.FitToEnclosure(TRUE, TRUE);
					end

				else if GetData('TEXT', theData) then
					begin
                        new(CStaticText(theContents));      { Altered by TCL Weaver 1.0 (5/9/90) }
                        itsContents := theContents;
						CStaticText(itsContents).IStaticText(itsScrollPane, SELF, 1, 1, 0, 0, sizELASTIC, sizELASTIC, 1000);
						itsContents.FitToEnclosure(TRUE, TRUE);
						SetRect(margin, 2, 2, -2, -2);
						itsContents.ChangeSize(margin, FALSE);
						CStaticText(itsContents).SetTextHandle(theData);
						DisposHandle(theData);
					end;

				if itsContents <> nil then
					begin
						itsScrollPane.InstallPanorama(itsContents);
						itsContents.Refresh;
					end;
			end;
	end;

end.