{****************************************************}
{ 		CDesktop.p}
{}
{		The Desktop Class}
{}
{	    	View which encompasses the entire screen. There is typically (always?) }
{	    	only one instance of CDesktop (or a subclass) and it is stored as a }
{	    	global variable (gDesktop). The Desktop is at the top of the visual }
{	    	hierarchy -- it encloses all other views and has no enclosure. The }
{	    	Desktop manages windows and has its own QuickDraw grafPort which may }
{	    	be used to draw directly on the screen (for example, to implement "zoom" }
{	    	rectangles to animate the opening and closing of windows, as done by }
{	    	the Finder). }
{}
{		SUPERCLASS = CView}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CDesktop;

interface

	uses
		Script, TCL;


implementation



{ **** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S **** }



{****************************************************}
{ IDesktop}
{}
{		Initialize a Desktop object}
{}
{****************************************************}

	procedure CDesktop.IDesktop (aSupervisor: CBureaucrat);

        var
            theWindows: CList;  { Altered by TCL Weaver 1.0 (5/9/90) }

	begin
		IView(nil, aSupervisor);		{ Initialize superclass			}

										{ Allocate space for a QD port	}

		macPort := GrafPtr(NewPtr(sizeof(GrafPort)));

		OpenPort(macPort);				{ Initialize QuickDraw port		}

		{ Make the port rectangle coincide with the bounding box of		}
		{ the gray region (entire screen excluding the menu bar). Also	}
		{ adjust the origin so that the local coords of this port are		}
		{ the same as global coords.											}

		bounds := GetGrayRgn^^.rgnBBox;
		PortSize(bounds.right - bounds.left, bounds.bottom - bounds.top);
		MovePortTo(bounds.left, bounds.top);
		SetOrigin(bounds.left, bounds.top);

		SetClip(GetGrayRgn);			{ Make the clipping and visible	}
										{   regions same as the grayRgn	}

		CopyRgn(GetGrayRgn, macPort^.visRgn);

		visible := TRUE;
		active := TRUE;
		wantsClicks := TRUE;

        new(theWindows);                { Initialize our window list        }
        itsWindows := theWindows;   { Altered by TCL Weaver 1.0 (5/9/90) }
		itsWindows.IList;
		topWindow := nil;
	end;


{****************************************************}
{ Free   (OVERRIDE)}
{}
{		Throw out objects created by a Desktop:  GrafPort and window list }
{}
{****************************************************}

	procedure CDesktop.Free;
	begin
		itsWindows.DisposeAll;
		ClosePort(macPort);
		DisposPtr(Ptr(macPort));
		inherited Free;
	end;


{****************************************************}
{	 DispatchClick {OVERRIDE) }
{}
{	        Respond to a mouse down. Determine where the mouse down occurred }
{	        and send the appropriate messages to the affected objects. The }
{	        where field of macEvent is in Global coordinates. }
{}
{****************************************************}

	procedure CDesktop.DispatchClick (var macEvent: EventRecord);
		var
			thePart: integer;			{ Location of mouse click }
			macWindow: WindowPtr;	{ Window where click occurred }
			theWindow: CWindow;		{ Corresponding window object }
			menuChoice: longint;		{ Selection from a menu }

	begin
		thePart := FindWindow(macEvent.where, macWindow);
		if (macWindow <> nil) and (thePart <> inSysWindow) then
			theWindow := CWindow(GetWRefCon(macWindow));

		case thePart of

			inDesk: 
				begin
					CountClicks(SELF, macEvent);
					DoClick(macEvent.where, macEvent.modifiers, macEvent.when);
				end;

			inMenuBar: 
				begin
					gBartender.UpdateAllMenus;
					menuChoice := MenuSelect(macEvent.where);
					if HiWord(menuChoice) <> 0 then
						begin
							gGopher.DoCommand(gBartender.FindCmdNumber(HiWord(menuChoice), LoWord(menuChoice)));
							HiliteMenu(0);
						end;
				end;

			inSysWindow: 
				SystemClick(macEvent, macWindow);

			inContent: 
				begin
					if (not theWindow.active) or (theWindow.floating and (WindowPtr(macWindow) <> FrontWindow)) then
						begin
							theWindow.Select;
							if not theWindow.actClick then
								Exit(DispatchClick)
							else
								theWindow.Activate;
						end;
					if theWindow.wantsClicks then
						begin
							UpdateWindows;
							theWindow.DispatchClick(macEvent);
						end
					else
						begin
							CountClicks(SELF, macEvent);
							DoClick(macEvent.where, macEvent.modifiers, macEvent.when);
						end;
				end;

			inDrag: 
				theWindow.Drag(macEvent);

			inGrow: 
				theWindow.Resize(macEvent);

			inGoAway: 
				if TrackGoAway(macWindow, macEvent.where) then
					theWindow.Close;

			inZoomIn, inZoomOut: 
				if TrackBox(macWindow, macEvent.where, thePart) then
					theWindow.Zoom(thePart);
		end;
	end;


{****************************************************}
{ DoMouseUp (OVERRIDE)}
{}
{		Respond to the mouse button being release. By default, ignore it.}
{}
{****************************************************}

	procedure CDesktop.DoMouseUp (macEvent: EventRecord);
	begin
	end;										{ Null Method						}


{****************************************************}
{ DispatchCursor}
{}
{		Adjust the shape of the cursor depending on the location of the mouse.}
{}
{****************************************************}

	procedure CDesktop.DispatchCursor (where: Point; mouseRgn: RgnHandle);
		var
			macWindow: WindowPtr;		{ Window containing the mouse		}
			theWindow: CWindow;			{ Corresponding window object		}
			thePart: integer;				{ Part of window containing mouse	}
			mbarRect: Rect;

	begin
		if gInBackground then			{ Do nothing if we are not the		}
			Exit(DispatchCursor);			{   foreground process				}

										{ Determine which window contains	}
										{   the mouse							}

		thePart := FindWindow(where, macWindow);

		if (thePart = inContent) or (thePart = inGrow) then

			{ Mouse is within the content	}
			{   region of a window. Check if	}
			{   that window is active.		}

			begin
				theWindow := CWindow(GetWRefCon(macWindow));
				if theWindow.IsActive then

			{ Window containing the mouse is	}
			{   active. Tell it to adjust the		}
			{   cursor shape.						}

					begin
						theWindow.DispatchCursor(where, mouseRgn);
						Exit(DispatchCursor);								{ Jump to end		}
					end
			end

		else if thePart = inMenuBar then
			begin
				SetCursor(arrow);				{ Use arrow cursor for menu bar	}
											{   and force it to be visible			}

				mbarRect := screenBits.bounds;
				mbarRect.bottom := mbarRect.top + GetMBarHeight;
				RectRgn(mouseRgn, mbarRect);
				Exit(DispatchCursor);				{ Jump to end						}
			end;

			{ All checks have failed. Mouse is	}
			{ in some insignificant area of		}
			{ the Desktop							}

		AdjustCursor(where, mouseRgn);
	end;


{****************************************************}
{ AdjustCursor (OVERRIDE)}
{}
{		Mouse is located where no other object wants to set the cursor shape.}
{		Adjust the shape to the standard arrow cursor}
{}
{****************************************************}

	procedure CDesktop.AdjustCursor (where: Point; mouseRgn: RgnHandle);
		var
			macWindow: WindowPeek;

	begin
		SetCursor(arrow);

		if topWindow <> nil then		{ A Window is active, so subtract	}
										{   its content region from the		}
										{   desktop								}
			begin
				macWindow := WindowPeek(topWindow.GetMacPort);
				DiffRgn(GetGrayRgn, WindowPeek(topWindow.GetMacPort)^.contRgn, mouseRgn);
			end
		else 								{ There's no active window, so		}
										{   cursor has the same shape over	}
										{   the entire desktop					}

			CopyRgn(GetGrayRgn, mouseRgn);
	end;


{****************************************************}
{ Show (OVERRIDE)}
{}
{		Make a Desktop visible by showing all its windows}
{}
{****************************************************}

	procedure CDesktop.Show;
	begin
		if not visible then
			begin
				visible := TRUE;
				itsWindows.DoForEach(Window_Show);
			end;
	end;


{****************************************************}
{ Hide (OVERRIDE)}
{}
{		Make a Desktop invisible by hiding all its windows}
{}
{****************************************************}

	procedure CDesktop.Hide;
	begin
		if visible then
			begin
				itsWindows.DoForEach(Window_Hide);
				visible := FALSE;
				if gGopher = SELF then
					gGopher := itsSupervisor;
			end;
	end;


{****************************************************}
{ Activate (OVERRIDE)}
{}
{		Activate a Desktop by activating its top window}
{}
{****************************************************}

	procedure CDesktop.Activate;
	begin
		if not active then
			begin
				active := TRUE;
				gGopher := SELF;
				if topWindow <> nil then
					topWindow.Activate;
			end;
	end;


{****************************************************}
{ Deactivate (OVERRIDE)}
{}
{		Deactivate a Desktop by deactivating its top window}
{}
{****************************************************}

	procedure CDesktop.Deactivate;
	begin
		if active then
			begin
				active := FALSE;
				if topWindow <> nil then
					topWindow.Deactivate;
				if gGopher = SELF then
					gGopher := itsSupervisor;
			end;
	end;


{****************************************************}
{ ReallyVisible  (OVERRIDE) }
{}
{		Check whether a Desktop is visible.  Desktop has no enclosure. }
{}
{****************************************************}

	function CDesktop.ReallyVisible: Boolean;
	begin
		ReallyVisible := visible;
	end;


{****************************************************}
{ AddWind}
{}
{		Add a window to the desktop}
{}
{****************************************************}

	procedure CDesktop.AddWind (theWindow: CWindow);
	begin
		itsWindows.Add(theWindow);
	end;


{****************************************************}
{ RemoveWind}
{}
{		Remove a window from the desktop}
{}
{****************************************************}

	procedure CDesktop.RemoveWind (theWindow: CWindow);
	begin
		if theWindow.IsVisible then
			HideWind(theWindow);					{ Hide window before discarding it	}
		itsWindows.Remove(theWindow);
	end;


{****************************************************}
{ SelectWind}
{}
{		Select a window by bringing it to the front and making it active}
{}
{****************************************************}

	procedure CDesktop.SelectWind (theWindow: CWindow);
	begin
		if theWindow = topWindow then
			begin								{ This is already the top window	}

				if IsSystemWindow(WindowPeek(FrontWindow)) then
					begin
											{ A DA is on top					}
											{   Make this the active window	}

						SelectWindow(theWindow.macPort)
					end
			end

		else
			begin							{ This is not the current top one	}
										{ Move this one to the front		}

				if not theWindow.IsVisible then
					begin
						BringToFront(theWindow.macPort);
						ShowWindow(theWindow.macPort);
					end
				else
					begin
						SelectWindow(theWindow.macPort);
					end;

				if topWindow <> nil then
					begin							{ Deactivate the former top		}
						topWindow.Deactivate;		{   regular window				}

						CDirector(topWindow.itsSupervisor).active := FALSE;
					end;
				itsWindows.BringFront(theWindow);
				topWindow := theWindow;				{ Mark it as the top regular		}
				if gInBackground then
					CDirector(topWindow.itsSupervisor).Activate;
			end;
	end;


{****************************************************}
{ ShowWind}
{}
{		Make a window visible on the desktop}
{}
{****************************************************}

	procedure CDesktop.ShowWind (theWindow: CWindow);
		var
			oldTop: CWindow;

	begin
		ShowHide(theWindow.macPort, TRUE);

		oldTop := topWindow;
		topWindow := CWindow(itsWindows.FirstSuccess(Window_IsVisible));
		if theWindow = topWindow then
			begin
				if oldTop <> nil then
					oldTop.Deactivate;
				topWindow.Activate;
			end;
	end;


{****************************************************}
{ HideWind}
{}
{		Make a window invisible on the desktop}
{}
{****************************************************}

	procedure CDesktop.HideWind (theWindow: CWindow);
	begin
		ShowHide(theWindow.macPort, FALSE);
		theWindow.Deactivate;
		if theWindow = topWindow then
			begin
				topWindow := CWindow(itsWindows.FirstSuccess(Window_IsVisible));
				if IsSystemWindow(WindowPeek(FrontWindow)) then
					begin
						SelectWindow(FrontWindow);
						if topWindow <> nil then
							CDirector(topWindow.itsSupervisor).active := TRUE;
					end
				else if topWindow <> nil then
					topWindow.Activate;
			end
	end;


{****************************************************}
{ DragWind}
{}
{		Drag a window on the desktop}
{}
{****************************************************}

	procedure CDesktop.DragWind (theWindow: CWindow; macEvent: EventRecord);
		var
			dragBox: Rect;				{ Where window may be dragged		}

	begin
		if (theWindow <> topWindow) and (BAND(macEvent.modifiers, cmdKey) = 0) then
			SelectWind(theWindow);

		dragBox := bounds;
		InsetRect(dragBox, DRAG_MARGIN, DRAG_MARGIN);
		DragWindow(theWindow.macPort, macEvent.where, dragBox);
	end;


{****************************************************}
{ UpdateWindows}
{}
{		Update contents of all windows}
{}
{****************************************************}

	procedure Window_Update (theWindow: CWindow);

{ Update contents of a window}

		var
			macWindow: WindowPeek;

	begin
		macWindow := WindowPeek(theWindow.GetMacPort);
		if not EmptyRgn(macWindow^.updateRgn) then
			theWindow.Update;
	end;


	procedure CDesktop.UpdateWindows;
	begin
		itsWindows.DoForEach(Window_Update);
	end;


{****************************************************}
{ GetTopWindow}
{}
{		Return the topmost window belonging to a Desktop (does not}
{		include DA windows)}
{}
{****************************************************}

	function CDesktop.GetTopWindow: CWindow;
	begin
		GetTopWindow := topWindow;					{ Return instance variable			}
	end;


{****************************************************}
{ GetBottomWindow }
{}
{		Return the Desktop's bottom-most window (not including DAs). }
{}
{****************************************************}

	function CDesktop.GetBottomWindow: CWindow;
	begin
		GetBottomWindow := CWindow(itsWindows.LastItem);
	end;


{****************************************************}
{ GetBounds}
{}
{		Return the bounds of a Desktop}
{}
{****************************************************}

	procedure CDesktop.GetBounds (var theBounds: Rect);
	begin
		theBounds := bounds;				{ Return copy of instance variable	}
	end;


{****************************************************}
{ GetAperture   (OVERRIDE) }
{}
{ 		Return the portion of the Desktop where drawing may occur. }
{}
{****************************************************}

	procedure CDesktop.GetAperture (var theAperture: Rect);
	begin
		theAperture := bounds;
	end;


{****************************************************}
{ Contains   (OVERRIDE) }
{}
{		Check whether a Desktop contains a specified point.  Point is }
{		specified in global coords }
{}
{****************************************************}

	function CDesktop.Contains (thePoint: Point): Boolean;
	begin
		Contains := PtInRect(thePoint, bounds);
	end;


{****************************************************}
{ HitSamePart }
{}
{		Check whether two hit points are close enough together to be }
{		part of a multiple click inside the Desktop. }
{}
{****************************************************}

	function CDesktop.HitSamePart (var pointA, pointB: Point): Boolean;
	begin
		HitSamePart := (abs(pointA.h - pointB.h) < REASONABLY_CLOSE) and (abs(pointA.v - pointB.v) < REASONABLY_CLOSE);
	end;


{****************************************************}
{ Prepare   (OVERRIDE) }
{}
{		Set current grafPort to the Desktop's port. }
{}
{****************************************************}

	procedure CDesktop.Prepare;
	begin
		SetPort(macPort);
	end;


{****************************************************}
{ Cleanup }
{}
{		No cleanup is necessary with this desktop. }
{}
{****************************************************}

	procedure CDesktop.Cleanup;
	begin
	end;


end.