{****************************************************}
{ 		CDocument.p}
{}
{		The Document Class}
{}
{	    	Abstract class which associates a Window with a File. This is the }
{	    	typical model used by most Mac applications, where the relevent }
{	    	information for an application is displayed to the user in a window }
{	    	and stored on disk as a file. Programmers must create their own }
{	    	subclass of CDocument if their program supports the standard Mac }
{	    	concept of creating a new document ("New" in the File menu) or }
{	    	opening an existing document ("Open..." in the File menu). }
{}
{		SUPERCLASS = CDirector}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CDocument;

interface

	uses
		Printing, TCL;


implementation



{ **** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S **** }



{****************************************************}
{ IDocument}
{}
{		Initialize a Document object}
{}
{****************************************************}

	procedure CDocument.IDocument (aSupervisor: CApplication; printable: Boolean);
		var
			macTPrint: THPrint;
			macTPrintP: TPPrint;
			temp: Boolean;
            thePrinter: CPrinter;       { Altered by TCL Weaver 1.0 (5/9/90) }

	begin
		IDirector(aSupervisor);		{ Initialize superclass			}

		itsMainPane := nil;
		itsFile := nil;
		lastTask := nil;
		undone := FALSE;
		dirty := FALSE;

		if printable then
			begin
                new(thePrinter);            { Altered by TCL Weaver 1.0 (5/9/90) }
                itsPrinter := thePrinter;
				temp := itsPrinter.IPrinter(SELF, nil);
				macTPrint := THPrint(itsPrinter.GetPrintRecord);
				macTPrintP := macTPrint^;
				pageWidth := macTPrintP^.prInfo.rPage.right;
				pageHeight := macTPrintP^.prInfo.rPage.bottom;
			end
		else
			begin
				itsPrinter := nil;
				pageWidth := STD_PAGE_WIDTH;
				pageHeight := STD_PAGE_HEIGHT;
			end;
	end;


{****************************************************}
{ Free (OVERRIDE)}
{}
{		Dispose of a Document}
{}
{****************************************************}

	procedure CDocument.Free;
	begin
		if itsFile <> nil then
			itsFile.Free;
		if lastTask <> nil then
			lastTask.Free;
		if itsPrinter <> nil then
			itsPrinter.Free;
		inherited Free;
	end;


{****************************************************}
{ Notify}
{}
{		Notification that a subordinate has completed a task}
{}
{****************************************************}

	procedure CDocument.Notify (theTask: CTask);
	begin
		if lastTask <> nil then
			lastTask.Free;
		lastTask := theTask;
		undone := FALSE;
		dirty := TRUE;
	end;


{****************************************************}
{ DoCommand (OVERRIDE)}
{}
{		Execute a command}
{}
{****************************************************}

	procedure CDocument.DoCommand (theCommand: longint);
		var
			temp: Boolean;

	begin
		case theCommand of

			cmdClose: 
				temp := Close(FALSE);

			cmdSave: 
				begin
					SetCursor(gWatchCursor^^);
					temp := DoSave;
				end;

			cmdSaveAs: 
				temp := DoSaveFileAs;

			cmdRevert: 
				begin
					PositionDialog('ALRT', ALRTrevert);
					if CautionAlert(ALRTrevert, nil) = OK then
						begin
							SetCursor(gWatchCursor^^);
							DoRevert;
						end;
				end;

			cmdPageSetup: 
				if itsPrinter <> nil then
					temp := itsPrinter.DoPageSetup;

			cmdPrint: 
				if itsPrinter <> nil then
					itsPrinter.DoPrint;

			cmdUndo: 
				if lastTask <> nil then
					begin
						if undone then
							lastTask.Redo
						else
							lastTask.Undo;
						undone := not undone;
						UpdateUndo;
					end;

			otherwise
				inherited DoCommand(theCommand);
		end;
	end;


{****************************************************}
{ UpdateMenus {OVERRIDE}
{}
{		Perform menu management tasks }
{}
{****************************************************}

	procedure CDocument.UpdateMenus;
	begin
		inherited UpdateMenus;

		gBartender.EnableCmd(cmdSaveAs);

		if dirty then
			begin
				gBartender.EnableCmd(cmdSave);
				if itsFile <> nil then
					gBartender.EnableCmd(cmdRevert);
			end;

		if itsPrinter <> nil then
			begin
				gBartender.EnableCmd(cmdPageSetup);
				gBartender.EnableCmd(cmdPrint);
			end;

		if lastTask <> nil then
			begin
				gBartender.EnableCmd(cmdUndo);
				UpdateUndo;
			end;
	end;


{****************************************************}
{ Close (OVERRIDE)}
{}
{		Close a Document}
{}
{****************************************************}

	function CDocument.Close (quitting: Boolean): Boolean;
		var
			tempBool: Boolean;
			tempErr: OSErr;

	begin
		if ConfirmClose(quitting) then
			begin
				if itsFile <> nil then
					tempErr := itsFile.Close;
				tempBool := inherited Close(quitting);
				Close := TRUE;
			end
		else
			Close := FALSE;
	end;


{****************************************************}
{ NewFile}
{}
{		Set up a document with a new file, usually in response the "New"}
{		command in the File menu.}
{}
{****************************************************}

	procedure CDocument.NewFile;
	begin
	end;


{****************************************************}
{ OpenFile}
{}
{		Open a file for a document, usually in response the "Open"}
{		command in the File menu.}
{}
{****************************************************}

	procedure CDocument.OpenFile (macSFReply: SFReply);
	begin
	end;


{****************************************************}
{ CloseWind (OVERRIDE)}
{}
{		A window belonging this document needs to be closed, usually in}
{		response to a click in its go away box or selecting the "Close"}
{		command in the File menu.}
{}
{****************************************************}

	procedure CDocument.CloseWind (theWindow: CWindow);
		var
			temp: Boolean;

	begin
		temp := Close(FALSE);
	end;



{**** P R I N T I N G    M E T H O D S ****}



{****************************************************}
{ PageCount }
{}
{	        Return the number of pages in a Document. }
{}
{****************************************************}

	function CDocument.PageCount: integer;
		var
			pixWidth, pixHeight: longint;

	begin
		if itsMainPane <> nil then
			begin
				itsMainPane.GetPixelExtent(pixWidth, pixHeight);
				PageCount := pixHeight div pageHeight + 1;
			end
		else
			PageCount := 0;
	end;


{****************************************************}
{ AboutToPrint}
{}
{		The specified range of pages is about to be printed. Make sure}
{		that the range is valid and make changes if it's not.}
{}
{		This method also gives a Document the chance to perform any necessary}
{		bookkeeping, such as repaginating, before a print cycle is started.}
{		This is also the time for the Document to request information about}
{		the page size from its Printer.}
{}
{****************************************************}

	procedure CDocument.AboutToPrint (var firstPage, lastPage: integer);
		var
			macTPrint: THPrint;
			macTPrintP: TPPrint;
			swap: integer;

	begin
		macTPrint := THPrint(itsPrinter.GetPrintRecord);
		macTPrintP := macTPrint^;
		pageWidth := macTPrintP^.prInfo.rPage.right;
		pageHeight := macTPrintP^.prInfo.rPage.bottom;

		if firstPage > lastPage then
			begin
				swap := lastPage;
				lastPage := firstPage;
				firstPage := swap;
			end;
										{ Can't print more pages than are	}
										{   contained in the document		}

		lastPage := Min(lastPage, PageCount);

		if itsMainPane <> nil then
			itsMainPane.AboutToPrint(firstPage, lastPage);

	end;


{****************************************************}
{ PrintPageOfDoc}
{}
{		Print a single page of the document. This method must NOT send}
{		messages to the Printer object. At this time, the print driver}
{		is open and waiting for information to be drawn. Requests for}
{		status information, such as page specifications, will foul up}
{		the state of the print driver.}
{}
{****************************************************}

	procedure CDocument.PrintPageOfDoc (pageNum: integer);
	begin
		if itsMainPane <> nil then
			itsMainPane.PrintPage(pageNum, pageWidth, pageHeight);
	end;


{****************************************************}
{ DonePrinting}
{}
{		Print loop has been completed}
{}
{****************************************************}

	procedure CDocument.DonePrinting;
	begin
		if itsMainPane <> nil then
			itsMainPane.DonePrinting;
	end;



{ **** F I L I N G   M E T H O D S **** }



{****************************************************}
{	 DoSave }
{}
{	        Save a file under the same name. Return TRUE if save was successful. }
{	        Subclasses must override this method. }
{}
{****************************************************}

	function CDocument.DoSave: Boolean;
	begin
	end;


{****************************************************}
{	 DoSaveAs }
{}
{	        Save a file under another name. Return TRUE if save was successful. }
{	        Subclasses must override this method. }
{}
{****************************************************}

	function CDocument.DoSaveAs (macSFReply: SFReply): Boolean;
	begin
	end;


{****************************************************}
{	 DoRevert }
{}
{	        Throw out the current contents of a document and revert to the }
{	        last saved version. Subclasses must override this method. }
{}
{****************************************************}

	procedure CDocument.DoRevert;
	begin
	end;


{****************************************************}
{	 DoSaveFileAs }
{}
{	        Respond to SaveAs menu command by posting a put file dialog }
{	        box and sending the DoSaveAs message if confirmed. }
{	        Return TRUE if save was successful. }
{}
{****************************************************}

	function CDocument.DoSaveFileAs: Boolean;
		var
			macSFReply: SFReply;

	begin
		PickFileName(macSFReply);
		if macSFReply.good then
			begin
				SetCursor(gWatchCursor^^);
				DoSaveFileAs := DoSaveAs(macSFReply);
			end
		else
			DoSaveFileAs := FALSE;
	end;


{****************************************************}
{	 PickFileName }
{}
{	        Put up a standard put file dialog box to allow the user to }
{	        specify a new name under which to save a copy of the document. }
{}
{****************************************************}

	procedure CDocument.PickFileName (var macSFReply: SFReply);
		var
			corner: Point;				{ Top left corner of dialog box }
			origName: Str255;			{ Default name for file }
			prompt: StringHandle;		{ Prompt string }

	begin
		FindDlogPosition('DLOG', putDlgID, corner);
		GetName(origName);
		prompt := GetString(STRprompt);
		CheckResource(Handle(prompt));
		MoveHHi(Handle(prompt));
		HLock(Handle(prompt));
		SFPPutFile(corner, prompt^^, origName, nil, macSFReply, putDlgID, nil);
		HUnlock(Handle(prompt));
	end;


{****************************************************}
{	 ConfirmClose }
{}
{	        Don't close a dirty document without giving user a chance to save }
{	        the changes }
{}
{****************************************************}

	function CDocument.ConfirmClose (quitting: Boolean): Boolean;
		var
			okToClose: Boolean;
			docName, status: Str255;
			response: integer;

	begin
		okToClose := TRUE;
		if dirty then
			begin
				if itsWindow <> nil then
					itsWindow.Update;
				GetName(docName);
				if quitting then
					GetIndString(status, STRcommon, strQUITTING)
				else
					GetIndString(status, STRcommon, strCLOSING);
				ParamText(docName, status, '', '');
				PositionDialog('ALRT', ALRTsaveChanges);
				InitCursor;
				response := Alert(ALRTsaveChanges, nil);
				if response = answerYES then
					begin
						SetCursor(gWatchCursor^^);
						okToClose := DoSave;
					end
				else if response = answerCANCEL then
					okToClose := FALSE;
			end;
		ConfirmClose := okToClose;
	end;


{****************************************************}
{ GetName}
{}
{		Return the name of a Document.}
{}
{****************************************************}

	procedure CDocument.GetName (var theName: Str255);
	begin
		if itsFile <> nil then				{ 1st choice: name of its file		}
			itsFile.GetName(theName)
		else if itsWindow <> nil then		{ 2nd choice: name of its window	}
			itsWindow.GetTitle(theName)
		else 									{ 3rd choice: NULL string			}
			theName := nullStr;
	end;


{****************************************************}
{	 UpdateUndo }
{}
{	        Update the text of the "Undo" command to reflect the current }
{	        status of undo/redo and the task name. }
{}
{****************************************************}

	procedure CDocument.UpdateUndo;
		var
			status, task: Str255;

	begin
		if active then
			begin
				if undone then
					GetIndString(status, STRcommon, strREDO)
				else
					GetIndString(status, STRcommon, strUNDO);

				GetIndString(task, STRtaskNames, lastTask.GetNameIndex);
				status := concat(status, task);
			end

		else
			GetIndString(status, STRcommon, strUNDO);

		gBartender.SetCmdText(cmdUndo, status);
	end;

end.