 {****************************************************}
{ 		CPane.p}
{}
{		The Pane Class}
{}
{	    	Abstract class for objects which may be displayed within windows. Panes }
{	    	can also respond to the mouse, keyboard, and commands. Subclasses must }
{	    	be created to display different kinds of information and implement }
{	    	different ways of responding to user actions. }
{}
{		SUPERCLASS = CView}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CPane;

interface

	uses
		TCL;


implementation



{ **** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S **** }



{****************************************************}
{ IPane}
{}
{		Initialize a Pane object}
{}
{****************************************************}

	procedure CPane.IPane (anEnclosure: CView; aSupervisor: CBureaucrat; aWidth, aHeight, aHEncl, aVEncl: integer; aHSizing, aVSizing: SizingOption);
	begin
		IView(anEnclosure, aSupervisor);		{ Initialize superclass			}

		visible := TRUE;
		active := TRUE;

		width := aWidth;
		height := aHeight;
		hEncl := aHEncl;
		vEncl := aVEncl;
		hSizing := aHSizing;
		vSizing := aVSizing;
		autoRefresh := TRUE;

		printClip := clipFRAME;

		IPaneX;		{ Extra initialization }
	end;


{****************************************************}
{ IViewTemp (OVERRIDE)}
{}
{		Initialize a Pane using a template}
{}
{****************************************************}

	procedure CPane.IViewTemp (anEnclosure: CView; aSupervisor: CBureaucrat; viewData: Ptr);
		var
			p: PaneTempP;

	begin
		p := PaneTempP(viewData);			{ Initialize superclass			}

		inherited IViewTemp(anEnclosure, aSupervisor, @p^.sViewTemp);

		width := p^.width;					{ Set instance vars from template	}
		height := p^.height;
		hEncl := p^.hEncl;
		vEncl := p^.vEncl;
		hSizing := SizingOption(p^.hSizing);
		vSizing := SizingOption(p^.vSizing);
		autoRefresh := p^.autoRefresh <> 0;
		printClip := ClipOption(p^.printClip);

		IPaneX;		{ Extra initialization				}
	end;


{****************************************************}
{ IPaneX {OVERRIDE}
{}
{ Extra initialization for a Pane }
{}
{****************************************************}

	procedure CPane.IPaneX;
	begin
		printing := FALSE;
		macPort := itsEnclosure.GetMacPort;
		CalcFrame;								{ Set up Frame coordinates		}
		CalcAperture;							{ Determine visible portion		}

		itsEnvironment := nil;
		itsEnclosure.AddSubview(SELF);		{ This is a subview of enclosure	}
	end;


{****************************************************}
{ Free (OVERRIDE)}
{}
{		Dispose of a Pane}
{}
{****************************************************}

	procedure CPane.Free;
	begin
		if itsEnvironment <> nil then
			itsEnvironment.Free;		{ Throw out drawing environment	}

		inherited Free;				{ Pass message on to superclass	}
	end;



{ **** A C C E S S I N G   M E T H O D S **** }



{****************************************************}
{	 SetFrameOrigin }
{}
{	        Set the coordinates of the top left corner of a Pane's frame. The }
{	        bottom right corner is adjusted according to the width and }
{	        height. The frame determines the local QuickDraw coordinate system }
{	        (called Frame coordinates) used for drawing the pane. }
{}
{****************************************************}

	procedure CPane.SetFrameOrigin (fLeft, fTop: integer);
	begin
		frame.left := fLeft;
		frame.top := fTop;
		frame.right := frame.left + width;
		frame.bottom := frame.top + height;
	end;


{****************************************************}
{ GetFrame}
{}
{		Return the frame of the pane}
{}
{****************************************************}

	procedure CPane.GetFrame (var theFrame: Rect);
	begin
		theFrame := frame;					{ Return copy of instance variable	}
	end;


{****************************************************}
{ GetLengths}
{}
{		Return the width and height of a Pane}
{}
{****************************************************}

	procedure CPane.GetLengths (var theWidth, theHeight: integer);
	begin
		theWidth := width;					{ Return instance variables		}
		theHeight := height;
	end;


{****************************************************}
{ GetOrigin}
{}
{		Return the origin of a Pane. Origin is the location of the}
{		top left corner of the Pane's window, expressed in the}
{		Pane's Frame coordinates.}
{}
{****************************************************}

	procedure CPane.GetOrigin (var theHOrigin, theVOrigin: longint);
	begin
		theHOrigin := hOrigin;
		theVOrigin := vOrigin;
	end;


{****************************************************}
{	 GetAperture {OVERRIDE) }
{}
{	        Return the portion of the pane where drawing may occur. Aperture }
{	        is in the pane's Frame coordinates. }
{}
{****************************************************}

	procedure CPane.GetAperture (var theAperture: Rect);
	begin
		theAperture := aperture;
	end;


{****************************************************}
{ Contains}
{}
{		Check whether a Pane contains a point, which is specified in the}
{		local coordinates of the window containing the Pane}
{}
{****************************************************}

	function CPane.Contains (thePoint: Point): Boolean;
	begin
		WindToFrame(thePoint);						{ Convert to Frame coordinates		}
		Contains := PtInRect(thePoint, aperture);	{ Is point inside aperture?	}
	end;


{****************************************************}
{	 ReallyVisible {OVERRIDE) }
{}
{	        Check whether a Pane and all its enclosures are visible. Also }
{	        make sure that the origin does not place the pane outside }
{	        valid QuickDraw coordinates. }
{}
{****************************************************}

	function CPane.ReallyVisible: Boolean;
	begin
		if visible and ((frame.left - hOrigin) >= MININT) and ((frame.right - hOrigin) <= MAXINT) and ((frame.top - vOrigin) >= MININT) and ((frame.bottom - vOrigin) <= MAXINT) then
			ReallyVisible := itsEnclosure.ReallyVisible
		else
			ReallyVisible := FALSE;
	end;


{****************************************************}
{ GetPixelExtent}
{}
{		Retrieve the dimensions of a Pane in pixels, which is the same as}
{		the width and height of its frame}
{}
{****************************************************}

	procedure CPane.GetPixelExtent (var hExtent, vExtent: longint);
	begin
		hExtent := width;
		vExtent := height;
	end;


{****************************************************}
{ SetPrintClip}
{}
{		Specify the clipping option used when printing}
{}
{****************************************************}

	procedure CPane.SetPrintClip (aPrintClip: ClipOption);
	begin
		printClip := aPrintClip;
	end;



{*** D I S P L A Y   M E T H O D S ***}



{****************************************************}
{	 Show {OVERRIDE) }
{}
{	        Make a Pane visible }
{}
{****************************************************}

	procedure CPane.Show;
	begin
		if not visible then
			begin
				inherited Show;
				Refresh;
			end;
	end;


{****************************************************}
{	 Hide {OVERRIDE) }
{}
{	        Make a Pane invisible }
{}
{****************************************************}

	procedure CPane.Hide;
	begin
		if visible then
			begin
				Refresh;
				inherited Hide;
			end;
	end;



{*** S I Z E   A N D   L O C A T I O N   M E T H O D S ***}



{****************************************************}
{ Place}
{}
{		Place the top left corner of a Pane at a location specified in}
{		the coordinates of its enclosure}
{}
{****************************************************}

	procedure CPane.Place (aHEncl, aVEncl: integer; redraw: Boolean);
		var
			hLocation: longint;
			vLocation: longint;
			hOffset: integer;
			vOffset: integer;

	begin
		hEncl := aHEncl;
		vEncl := aVEncl;
										{ Determine location of pane in	}
										{   Window coords				}

		itsEnclosure.SubpaneLocation(hEncl, vEncl, hLocation, vLocation);

		hOffset := hOrigin - frame.left + hLocation;
		vOffset := vOrigin - frame.top + vLocation;

		Offset(hOffset, vOffset, redraw);
	end;


{****************************************************}
{ Offset}
{}
{		Translate the location of the Pane within its enclosure}
{}
{****************************************************}

	procedure Pane_EnclosureMoved (thePane: CPane; theOffset: Ptr);

{	          Tell a Pane that its enclosure has moved. Send Offset message	}
{	          with redraw set to FALSE, since enclosure will have already		}
{	          taken care of adding the appropriate areas to the update region. 	}

	begin
		thePane.Offset(PointPtr(theOffset)^.h, PointPtr(theOffset)^.v, FALSE);
	end;


	procedure CPane.Offset (hOffset, vOffset: integer; redraw: Boolean);
		var
			theOffset: Point;

	begin
		if redraw then						{ Force redraw of area covered by	}
			Refresh;								{ current location of pane				}

		hOrigin := hOrigin - hOffset;		{ Change location of pane				}
		vOrigin := vOrigin - vOffset;

		CalcAperture;						{ Visible portion of pane could change as a	}
											{ result of a shift in its location				}

		if redraw then
			Refresh;								{ Redraw pane at its new location			}

		if itsSubviews <> nil then		{ Inform subviews of the move so they can	}
			begin								{ adjust their locations							}
				theOffset.h := hOffset;
				theOffset.v := vOffset;
				itsSubviews.DoForEach1(Pane_EnclosureMoved, @theOffset);
			end;
	end;


{****************************************************}
{ ChangeSize}
{}
{	        Change the size of a Pane. The delta rectangle parameter specifies }
{	        how each side of the pane should change. Positive values mean a }
{	        move to the right or down, negative values to the left or up. }
{}
{****************************************************}

	procedure CPane.ChangeSize (delta: Rect; redraw: Boolean);
		var
			oldAperture: Rect;
			temp: Boolean;

	begin
		oldAperture := aperture;		{ Save old aperture    						}

		if redraw then					{ Force redraw of area currently occupied	}
			Refresh;							{ by the pane									}

		ResizeFrame(delta);				{ Change dimensions of pane					}

		CalcAperture;					{ determine new aperture					}

		if redraw then					{ Force draw of pane at new size			}
			Refresh;

	{ If autoRefresh is FALSE (and other safety checks), we do not want to	}
	{ redraw the area which is common to the old and new pane size.  Note	}
	{ that the Refresh method will have set the port for us already.			}

		if (not autoRefresh) and ReallyVisible then
			begin
				temp := SectRect(aperture, oldAperture, oldAperture);
				ValidRect(oldAperture);
			end;

		if itsSubviews <> nil then
			itsSubviews.DoForEach1(Pane_AdjustToEnclosure, @delta);
	end;


{****************************************************}
{ AdjustToEnclosure}
{}
{	        Adjust size and/or location of a Pane after its enclosure's size }
{	        has changed. The deltaEncl rectangle parameter specifies how each }
{	        side of the enclosure has changed. Positive values mean a move to }
{	        the right or down, negative values to the left or up. }
{}
{****************************************************}

	procedure CPane.AdjustToEnclosure (deltaEncl: Rect);
		var
			delta: Rect;
			hOffset, vOffset: integer;
			moved, sized: Boolean;

	begin
		hOffset := 0;
		vOffset := 0;
		moved := FALSE;
		sized := FALSE;

		SetRect(delta, 0, 0, 0, 0);
		AdjustHoriz(deltaEncl, delta, hOffset, moved, sized);
		AdjustVert(deltaEncl, delta, vOffset, moved, sized);

		if moved then
			if sized then			{ Pane must be moved and sized	}
				begin
					Offset(hOffset, vOffset, TRUE);
					ChangeSize(delta, TRUE);
				end
			else
				Offset(hOffset, vOffset, TRUE)

		else if sized then
			ChangeSize(delta, TRUE)
		else
			CalcAperture;			{ Pane stays the same, but change in enclosure may	}
								{ change its aperture.										}

	end;


{****************************************************}
{	 AdjustHoriz }
{}
{	        Adjust the horizontal size and/or location of a Pane }
{}
{****************************************************}

	procedure CPane.AdjustHoriz (deltaEncl: Rect; var delta: Rect; var theOffset: integer; var moved, sized: Boolean);
	begin
		case hSizing of

			sizFIXEDLEFT: 
				begin
					theOffset := deltaEncl.left;
					if theOffset <> 0 then
						moved := TRUE;
				end;

			sizFIXEDRIGHT: 
				begin
					theOffset := deltaEncl.right;
					if theOffset <> 0 then
						moved := TRUE;
				end;

			sizELASTIC: 
				begin
					delta.left := deltaEncl.left;
					delta.right := deltaEncl.right;
					if (delta.left <> 0) or (delta.right <> 0) then
						sized := TRUE;
				end;

			otherwise
				;
		end;
	end;


{****************************************************}
{	 AdjustVert }
{}
{	        Adjust the vertical size and/or location of a Pane }
{}
{****************************************************}

	procedure CPane.AdjustVert (deltaEncl: Rect; var delta: Rect; var theOffset: integer; var moved, sized: Boolean);
	begin
		case vSizing of

			sizFIXEDTOP: 
				begin
					theOffset := deltaEncl.top;
					if theOffset <> 0 then
						moved := TRUE;
				end;

			sizFIXEDBOTTOM: 
				begin
					theOffset := deltaEncl.bottom;
					if theOffset <> 0 then
						moved := TRUE;
				end;

			sizELASTIC: 
				begin
					delta.top := deltaEncl.top;
					delta.bottom := deltaEncl.bottom;
					if (delta.top <> 0) or (delta.bottom <> 0) then
						sized := TRUE;
				end;

			otherwise
				;
		end;
	end;


{****************************************************}
{	 EnclosureScrolled }
{}
{	        Adjust location of a Pane after its enclosure has scrolled. This }
{	        only matters if the Pane is sticky in the direction of movement. }
{}
{****************************************************}

	procedure CPane.EnclosureScrolled (hOffset, vOffset: integer);
	begin
		if hSizing <> sizFIXEDSTICKY then
			hOffset := 0;
		if vSizing <> sizFIXEDSTICKY then
			vOffset := 0;
		if (hOffset <> 0) or (vOffset <> 0) then
			Offset(-hOffset, -vOffset, FALSE);
	end;



{ **** A D A P T I N G   M E T H O D S **** }



{****************************************************}
{ FitToEnclosure}
{}
{		Make the frame of a Pane "fit" to the interior of its enclosure in}
{		the horizontal and/or vertical directions. For a horizontal fit,}
{		the Pane's width and left edge are changed to coincide with}
{		those of its enclosure's interior. Similarly for a vertical fit.}
{		Pane is NOT automatically redrawn.}
{}
{****************************************************}

	procedure CPane.FitToEnclosure (horizFit, vertFit: Boolean);
		var
			interior: Rect;
			hOffset: integer;
			vOffset: integer;
			delta: Rect;

	begin
		hOffset := 0;
		vOffset := 0;

		itsEnclosure.GetInterior(interior);
		EnclToFrameR(interior);

		delta.left := 0;
		delta.top := 0;
		delta.right := 0;
		delta.bottom := 0;

		if horizFit then
			begin
				hOffset := interior.left - frame.left;
				delta.right := (interior.right - interior.left) - width;
			end;

		if vertFit then
			begin
				vOffset := interior.top - frame.top;
				delta.bottom := (interior.bottom - interior.top) - height;
			end;

		Offset(hOffset, vOffset, FALSE);
		ChangeSize(delta, FALSE);
	end;


{****************************************************}
{ FitToEnclFrame}
{}
{		Make the frame of a Pane "fit" to the frame of its enclosure in}
{		the horizontal and/or vertical directions. For a horizontal fit,}
{		the Pane's width and left edge are changed to coincide with}
{		those of its enclosure's frame. Similarly for a vertical fit.}
{		Pane is NOT automatically redrawn.}
{}
{****************************************************}

	procedure CPane.FitToEnclFrame (horizFit, vertFit: Boolean);
		var
			enclFrame: Rect;
			hOffset: integer;
			vOffset: integer;
			delta: Rect;

	begin
		hOffset := 0;
		vOffset := 0;

		itsEnclosure.GetFrame(enclFrame);
		EnclToFrameR(enclFrame);

		delta.left := 0;
		delta.top := 0;
		delta.right := 0;
		delta.bottom := 0;

		if horizFit then
			begin
				hOffset := enclFrame.left - frame.left;
				delta.right := (enclFrame.right - enclFrame.left) - width;
			end;

		if vertFit then
			begin
				vOffset := enclFrame.top - frame.top;
				delta.bottom := (enclFrame.bottom - enclFrame.top) - height;
			end;

		Offset(hOffset, vOffset, FALSE);
		ChangeSize(delta, FALSE);
	end;


{****************************************************}
{ CenterWithinEnclosure}
{}
{		Center a Pane's frame within the interior of its enclosure's frame,}
{		horizontally and/or vertically. Only the location of the}
{		Pane is changed; its size is not altered. Pane is NOT automatically}
{		redrawn.}
{}
{****************************************************}

	procedure CPane.CenterWithinEnclosure (horizCenter, vertCenter: Boolean);
		var
			interior: Rect;
			hOffset: integer;
			vOffset: integer;

	begin
		hOffset := 0;
		vOffset := 0;

		itsEnclosure.GetInterior(interior);
		EnclToFrameR(interior);

		if horizCenter then
			hOffset := interior.left - frame.left + (interior.right - interior.left - width) div 2;

		if vertCenter then
			vOffset := interior.top - frame.top + (interior.bottom - interior.top - height) div 2;

		Offset(hOffset, vOffset, FALSE);
	end;



{*** D R A W I N G   M E T H O D S ***}



{****************************************************}
{	 Draw }
{}
{	   	Draw the contents of the Pane. The area parameter, specified in }
{	   	the pane's Frame coordinates, indicates the portion of the Pane }
{	    	which needs to be drawn. Subclasses of CPane which display}
{		data (rather than just manage subviews) must override this method.}
{}
{****************************************************}

	procedure CPane.Draw (var area: Rect);
	begin
	end;


{****************************************************}
{ DrawAll}
{}
{		Draw a Pane and all of its subviews. The port must be prepared}
{		ahead of time.}
{}
{****************************************************}

	procedure CPane.DrawAll (var area: Rect);
		var
			temp: Boolean;

	begin
		Prepare;
		if not printing then
			begin
				temp := SectRect(area, aperture, area);
				ClipRect(area);
			end;

		Draw(area);

		if itsSubviews <> nil then
			itsSubviews.DoForEach1(Pane_Draw, @area);
	end;


{****************************************************}
{	 Refresh }
{}
{	        Force a redraw of an entire Pane. }
{}
{****************************************************}

	procedure CPane.Refresh;
		var
			pFrame: Rect;

	begin
		pFrame := frame;
		RefreshRect(pFrame);
	end;


{****************************************************}
{	 RefreshRect }
{}
{	        Force a redraw of an area by adding it to the update region }
{	        of its window. Drawing will not actually take place until the next }
{	        update event is processed. We must be sure to include only the }
{	        truly visible portion of the area -- the part which shows thru }
{	        the aperture of its enclosure. }
{}
{****************************************************}

	procedure CPane.RefreshRect (area: Rect);
		var
			enclAperture: Rect;
			temp: Boolean;

	begin
		if ReallyVisible then
			begin
				SetPort(macPort);
				SetOrigin(integer(hOrigin), integer(vOrigin));
				itsEnclosure.GetAperture(enclAperture);
				EnclToFrameR(enclAperture);
				temp := SectRect(area, enclAperture, enclAperture);
				InvalRect(enclAperture);
			end;
	end;



{ **** P R I N T I N G   M E T H O D S **** }



{****************************************************}
{ AboutToPrint}
{}
{		The specified range of pages is about to be printed.}
{}
{****************************************************}

	procedure CPane.AboutToPrint (var firstPage, lastPage: integer);
	begin
		printing := TRUE;
	end;


{****************************************************}
{	 PrintPage }
{}
{	        Print the specified page. This method ignores the page number and }
{	        simply draws the entire pane. Subclasses which support multi-page }
{	        panes must override this method. }
{}
{****************************************************}

	procedure CPane.PrintPage (pageNum, pageWidth, pageHeight: integer);
		var
			area: Rect;

	begin
		area.left := frame.left;
		area.top := frame.top;
		area.right := area.left + pageWidth;
		area.bottom := area.top + pageHeight;
		DrawAll(area);
	end;


{****************************************************}
{ DonePrinting}
{}
{		Print loop has been completed. Perform any necessary clean-up.}
{}
{****************************************************}

	procedure CPane.DonePrinting;
	begin
		printing := FALSE;
	end;


{****************************************************}
{ PrepareToPrint}
{}
{		Set up coordinate system and clipping region for printing. This}
{		method prepares the port so that only the portion of the pane}
{		which is visible on the screen is printed.}
{}
{****************************************************}

	procedure CPane.PrepareToPrint;
		var
			clipR: Rect;

	begin
		SetOrigin(integer(hOrigin), integer(vOrigin));

		case printClip of

			clipAPERTURE: 
				clipR := aperture;

			clipFRAME: 
				clipR := frame;

			clipPAGE: 
				clipR := thePort^.portRect;
		end;

		ClipRect(clipR);
	end;



{ **** C A L I B R A T I N G   M E T H O D S **** }



{****************************************************}
{	 Prepare }
{}
{	        Prepare for drawing the Pane by setting the port, establishing }
{	        the correct coordinate system, clipping the the proper area, and }
{	        restoring its drawing environment. }
{}
{****************************************************}

	procedure CPane.Prepare;
		var
			tempRect: Rect;		{ ClipRect may move memory	}

	begin
		if printing then
			PrepareToPrint
		else
			begin
				SetPort(macPort);
				SetOrigin(integer(hOrigin), integer(vOrigin));	{ Set to Frame coords	}
				tempRect := aperture;
				ClipRect(tempRect);				{ Make sure that drawing occurs only 	}
											{ inside the visible portion of the Pane	}
			end;

		if itsEnvironment <> nil then
			itsEnvironment.Restore;
	end;


{****************************************************}
{ RestoreEnvironment}
{}
{		Restore the drawing environment of a Pane}
{}
{****************************************************}

	procedure CPane.RestoreEnvironment;
	begin
		if itsEnvironment <> nil then
			itsEnvironment.Restore;
	end;


{****************************************************}
{	 CalcFrame }
{}
{	        Determine the coordinates of the Pane's frame and origin based on }
{	        the width, height, hEncl, vEncl. These instance variables must be }
{	        set before invoking this method. }
{}
{	        The frame is set to have its top left corner at (0, 0). The defines }
{	        the Frame coordinates of the Pane. Subclasses must override this }
{	        method if they use a different Frame coordinate system. }
{}
{****************************************************}

	procedure CPane.CalcFrame;
		var
			hLocation, vLocation: longint;

	begin
		frame.left := 0;
		frame.top := 0;
		frame.right := width;
		frame.bottom := height;

	{ Determine location of pane in Window coords	}

		itsEnclosure.SubpaneLocation(hEncl, vEncl, hLocation, vLocation);

	{ Origin maps between Frame and Window coords.  Origin is the top left	}
	{ corner of the window expressed in Frame coords.							}

		hOrigin := frame.left - hLocation;
		vOrigin := frame.top - vLocation;
	end;


{****************************************************}
{	 ResizeFrame }
{}
{	        Adjust a Pane's frame when the size of the pane changes. The delta }
{	        rectangle parameter specifies the change in location of each side }
{	        of the Pane. Positive values mean a move to the right or down; }
{	        negative values mean a move to the left or up. }
{}
{	        This method always adjusts the width and height to the new size }
{	        and puts the top left of the frame at (0, 0). This will cause a }
{	        shift in the origin if the left or top sides of the pane move. }
{}
{****************************************************}

	procedure CPane.ResizeFrame (delta: Rect);
	begin
		width := width + delta.right - delta.left;			{ Adjust dimensions of pane		}
		height := height + delta.bottom - delta.top;
		frame.top := 0;										{ Put top left of frame at (0,0)	}
		frame.left := 0;
		frame.right := width;
		frame.bottom := height;
		hOrigin := hOrigin - delta.left;			{ Origin moves with top left corner of pane	}
		vOrigin := vOrigin - delta.top;
	end;


{****************************************************}
{	 CalcAperture {OVERRIDE) }
{}
{	        Determine the area of the pane where drawing may occur. This is }
{	        the area within the frame that is not obscured by the bounds }
{	        of enclosing views. The aperture is in Frame coordinates. }
{}
{****************************************************}

	procedure Pane_CalcAperture (thePane: CPane);
	begin
		thePane.CalcAperture;
	end;

	procedure CPane.CalcAperture;
		var
			enclAperture: Rect;
			temp: Boolean;

	begin
		itsEnclosure.GetAperture(enclAperture);
		EnclToFrameR(enclAperture);
		temp := SectRect(frame, enclAperture, aperture);	{ Aperture is the frame	}
																	{ restricted by the 			}
																	{ enclosure's aperture.		}
		if itsSubviews <> nil then
			itsSubviews.DoForEach(Pane_CalcAperture);			{ Pass it on to subviews	}
	end;



{ * * * * C O O R D I N A T E    T R A N S F O R M    M E T H O D S * * * * }



{****************************************************}
{	 WindToFrame }
{}
{	        Convert a point from Window coords to the Frame coords of a Pane }
{}
{****************************************************}

	procedure CPane.WindToFrame (var thePoint: Point);
	begin
		thePoint.h := thePoint.h + integer(hOrigin);
		thePoint.v := thePoint.v + integer(vOrigin);
	end;


{****************************************************}
{	 WindToFrameR }
{}
{	        Convert a rectangle from Window coords to the Frame coords of a Pane }
{}
{****************************************************}

	procedure CPane.WindToFrameR (var theRect: Rect);
	begin
		OffsetRect(theRect, hOrigin, vOrigin);
	end;


{****************************************************}
{	 FrameToWind }
{}
{	        Convert a point from the Frame coords of a Pane to Window coords }
{}
{****************************************************}

	procedure CPane.FrameToWind (var thePoint: Point);
	begin
		thePoint.h := thePoint.h - hOrigin;
		thePoint.v := thePoint.v - vOrigin;
	end;


{****************************************************}
{	 FrameToWindR }
{}
{	        Convert a rectangle from the Frame coords of a Pane to Window coords }
{}
{****************************************************}

	procedure CPane.FrameToWindR (var theRect: Rect);
	begin
		OffsetRect(theRect, -hOrigin, -vOrigin);
	end;


{****************************************************}
{	 EnclToFrame }
{}
{	        Convert a point from the Frame coords of the enclosure to the }
{	        Frame coords of the Pane }
{}
{****************************************************}

	procedure CPane.EnclToFrame (var thePoint: Point);
		var
			enclHOrigin, enclVOrigin: longint;

	begin
		itsEnclosure.GetOrigin(enclHOrigin, enclVOrigin);
		thePoint.h := thePoint.h + hOrigin - enclHOrigin;
		thePoint.v := thePoint.v + vOrigin - enclVOrigin;
	end;


{****************************************************}
{	 EnclToFrameR }
{}
{	        Convert a rectangle from the Frame coords of the enclosure to the }
{	        Frame coords of the Pane }
{}
{****************************************************}

	procedure CPane.EnclToFrameR (var theRect: Rect);
		var
			enclHOrigin, enclVOrigin: longint;

	begin
		itsEnclosure.GetOrigin(enclHOrigin, enclVOrigin);
		OffsetRect(theRect, hOrigin - enclHOrigin, vOrigin - enclVOrigin);
	end;


{****************************************************}
{	 FrameToEncl }
{}
{	        Convert a point from the Frame coords of a Pane to the }
{	        Frame coords of its enclosure }
{}
{****************************************************}

	procedure CPane.FrameToEncl (var thePoint: Point);
		var
			enclHOrigin, enclVOrigin: longint;

	begin
		itsEnclosure.GetOrigin(enclHOrigin, enclVOrigin);
		thePoint.h := thePoint.h + enclHOrigin - hOrigin;
		thePoint.v := thePoint.v + enclVOrigin - vOrigin;
	end;


{****************************************************}
{	 FrameToEnclR }
{}
{	        Convert a rectangle from the Frame coords of a Pane to the }
{	        Frame coords of its enclosure }
{}
{****************************************************}

	procedure CPane.FrameToEnclR (var theRect: Rect);
		var
			enclHOrigin, enclVOrigin: longint;

	begin
		itsEnclosure.GetOrigin(enclHOrigin, enclVOrigin);
		OffsetRect(theRect, enclHOrigin - hOrigin, enclVOrigin - vOrigin);
	end;


{****************************************************}
{ FrameToGlobalR (OVERRIDE)}
{}
{		Convert a rectangle from Frame to Global coordinates}
{}
{****************************************************}

	procedure CPane.FrameToGlobalR (var theRect: Rect);
		var
			theOffset: Point;

	begin
    { Altered by TCL Weaver 1.0 (4/27/90) }

        theOffset := topLeft(WindowPeek(macPort)^.contRgn^^.rgnBBox);

        OffsetRect(theRect, theOffset.h - hOrigin, theOffset.v - vOrigin);
	end;


end.