{****************************************************}
{ 		CScrollPane.p}
{}
{		The ScrollPane Class}
{}
{		A pane for managing a Panorama and a horizontal and/or vertical }
{		scroll bar. Handles all communication between a Panorama and }
{		scroll bars so that they are always in synch. ScrollPane tells }
{		the Panorama how much to scroll when there are clicks in a }
{		scroll bar and adjust the scroll bars when the Panorama changes. }
{}
{		SUPERCLASS = CPane}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CScrollPane;

interface

	uses
		TCL;

	{ Scroll action functions called through pointers }

	procedure SBarActionProc (macControl: ControlHandle; whichPart: integer);
	procedure SBarThumbFunc (theSBar: CScrollBar; delta: integer);


implementation


{****************************************************}
{ SBarActionProc }
{}
{		Function called continuously while user holds mouse down within}
{		a fixed part of a scroll bar}
{}
{****************************************************}

	procedure SBarActionProc (macControl: ControlHandle; whichPart: integer);
		var
			theSBar: CScrollBar;				{ Associated scroll bar object		}

	begin
		if whichPart <> 0 then
			begin
				theSBar := CScrollBar(GetCRefCon(macControl));
				if theSBar.theOrientation = HORIZONTAL then
					CScrollPane(theSBar.itsSupervisor).DoHorizScroll(whichPart)
				else
					CScrollPane(theSBar.itsSupervisor).DoVertScroll(whichPart);
			end;
	end;


{****************************************************}
{ SBarThumbFunc }
{}
{		Function called after user drags the thumb of a scroll bar}
{}
{****************************************************}

	procedure SBarThumbFunc (theSBar: CScrollBar; delta: integer);
	begin
		if theSBar.theOrientation = HORIZONTAL then
			CScrollPane(theSBar.itsSupervisor).DoThumbDrag(delta, 0)
		else
			CScrollPane(theSBar.itsSupervisor).DoThumbDrag(0, delta);
	end;



{ **** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S **** }



{****************************************************}
{ IScrollPane}
{}
{		Initialize a ScrollPane object}
{}
{****************************************************}

	procedure CScrollPane.IScrollPane (anEnclosure: CView; aSupervisor: CBureaucrat; aWidth, aHeight, aHEncl, aVEncl: integer; aHSizing, aVSizing: SizingOption; hasHoriz, hasVert, hasSizeBox: Boolean);
		var
			shrinkage: integer;

	begin
		IPane(anEnclosure, aSupervisor, aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);

		hStep := 1;
		vStep := 1;
		hOverlap := 1;
		vOverlap := 1;

		IScrollPaneX(hasHoriz, hasVert, hasSizeBox);
	end;


{****************************************************}
{IViewTemp  (OVERRIDE)}
{}
{		Initialize a ScrollPane using a template.}
{}
{****************************************************}

	procedure CScrollPane.IViewTemp (anEnclosure: CView; aSupervisor: CBureaucrat; viewData: Ptr);
		var
			p: ScrollPaneTempP;

	begin
		p := ScrollPaneTempP(viewData);
		inherited IViewTemp(anEnclosure, aSupervisor, @p^.sPaneTemp);

		hStep := p^.hStep;
		vStep := p^.vStep;
		hOverlap := p^.hOverlap;
		vOverlap := p^.vOverlap;

		IScrollPaneX(p^.hasHoriz <> 0, p^.hasVert <> 0, p^.hasSizeBox <> 0);
	end;


{****************************************************}
{IScrollPaneX}
{}
{		Extra initialization.}
{}
{****************************************************}

	procedure CScrollPane.IScrollPaneX (hasHoriz, hasVert, hasSizeBox: Boolean);
		var
			shrinkage: integer;
            theHorizSBar, theVertSBar: CScrollBar;  { Altered by TCL Weaver 1.0 (5/9/90) }
            theSizeBox: CSizeBox;

	begin
		wantsClicks := TRUE;

		if hasSizeBox or (hasHoriz and hasVert) then
			shrinkage := SBARSIZE1
		else
			shrinkage := 0;

		if hasHoriz then
			begin
                new(theHorizSBar);              { Altered by TCL Weaver 1.0 (5/9/90) }
                itsHorizSBar := theHorizSBar;
				itsHorizSBar.IScrollBar(SELF, SELF, HORIZONTAL, width - shrinkage, frame.left, frame.bottom - SBARSIZE);
				itsHorizSBar.SetActionProc(@SBarActionProc);
				itsHorizSBar.SetThumbFunc(@SBarThumbFunc);
			end
		else
			itsHorizSBar := nil;

		if hasVert then
			begin
                new(theVertSBar);               { Altered by TCL Weaver 1.0 (5/9/90) }
                itsVertSBar := theVertSBar;
				itsVertSBar.IScrollBar(SELF, SELF, VERTICAL, height - shrinkage, frame.right - SBARSIZE, frame.top);
				itsVertSBar.SetActionProc(@SBarActionProc);
				itsVertSBar.SetThumbFunc(@SBarThumbFunc);
			end
		else
			itsVertSBar := nil;

		if hasSizeBox then
			begin
                new(theSizeBox);                { Altered by TCL Weaver 1.0 (5/9/90) }
                itsSizeBox := theSizeBox;
				itsSizeBox.ISizeBox(SELF, SELF);
			end
		else
			itsSizeBox := nil;

		itsPanorama := nil;
	end;



{ **** A C C E S S I N G   M E T H O D S **** }



{****************************************************}
{ InstallPanorama}
{}
{		Install a Panorama within a ScrollPane}
{}
{****************************************************}

	procedure CScrollPane.InstallPanorama (aPanorama: CPanorama);
	begin
		itsPanorama := aPanorama;
		itsPanorama.SetScrollPane(SELF);
		AdjustScrollMax;
		Calibrate;
	end;


{****************************************************}
{ SetSteps}
{}
{	        Set the horizontal and vertical steps for scrolling when clicking }
{	        in one of the arrows at the end of a scroll bar. Normally, a }
{	        step is one unit, where a unit is defined by the Panorama. }
{}
{****************************************************}

	procedure CScrollPane.SetSteps (aHStep, aVStep: integer);
	begin
		hStep := aHStep;						{ Set instance variables			}
		vStep := aVStep;
	end;


{****************************************************}
{GetSteps}
{}
{		Get the horizontal and vertical steps for scrolling when clicking}
{		in one of the arrows at the end of a scroll bar.}
{}
{****************************************************}

	procedure CScrollPane.GetSteps (var theHStep, theVStep: integer);
	begin
		theHStep := hStep;
		theVStep := vStep;
	end;


{****************************************************}
{ SetOverlaps}
{}
{	        Set the horizontal and vertical overlaps for page scrolling. The }
{	        overlap determines how much "context" remains when scrolling by }
{	        a screenful. Normally, the overlap is one unit, where a unit is }
{	        defined by the Panorama. }
{}
{****************************************************}

	procedure CScrollPane.SetOverlaps (aHOverlap, aVOverlap: integer);
	begin
		hOverlap := aHOverlap;				{ Set instance variables			}
		vOverlap := aVOverlap;
	end;


{****************************************************}
{ GetInterior (OVERRIDE)}
{}
{    Retrieve the interior of a ScrollPane . Interior excludes the area of the }
{	frame occupied by the scroll bars. }
{}
{****************************************************}

	procedure CScrollPane.GetInterior (var theInterior: Rect);
	begin
		theInterior := frame;				{ Start off with the frame		}

		theInterior.left := theInterior.left + 1;
		theInterior.top := theInterior.top + 1;

		if itsHorizSBar <> nil then			{ Subtract horizontal scroll bar	}

			theInterior.bottom := theInterior.bottom - SBARSIZE;

		if itsVertSBar <> nil then			{ Subtract vertical scroll bar	}

			theInterior.right := theInterior.right - SBARSIZE;
	end;



{ **** S C R O L L   B A R   M A I N T E N A N C E   M E T H O D S **** }



{****************************************************}
{ AdjustScrollMax}
{}
{		Adjust the maximum value of the scroll bars to conform to the}
{		size of the bounds and frame of the Panorama}
{}
{****************************************************}

	procedure CScrollPane.AdjustScrollMax;
		var
			newMax: integer;
			hPos, vPos: longint;
			hFSpan, vFSpan: integer;

	begin
		if itsPanorama = nil then
			Exit(AdjustScrollMax);

		itsPanorama.GetExtent(hPos, vPos);
		hExtent := hPos;
		vExtent := vPos;

		itsPanorama.GetFrameSpan(hFSpan, vFSpan);
		hSpan := hFSpan;
		vSpan := vFSpan;

		hUnit := hExtent div MAXINT + 1;
		vUnit := vExtent div MAXINT + 1;

		itsPanorama.GetFramePosition(hPos, vPos);

		if itsHorizSBar <> nil then
			itsHorizSBar.SetMaxValue(Max((hExtent - hSpan), hPos) div hUnit);

		if itsVertSBar <> nil then
			itsVertSBar.SetMaxValue(Max((vExtent - vSpan), vPos) div vUnit);
	end;


{****************************************************}
{	 Calibrate }
{}
{	        Adjust scroll bar setting when the position of the frame of the }
{	        ScrollPane's panorama changes }
{}
{****************************************************}

	procedure CScrollPane.Calibrate;
		var
			hPos, vPos: longint;

	begin
		if itsPanorama = nil then
			Exit(Calibrate);

		itsPanorama.GetFramePosition(hPos, vPos);

		if itsHorizSBar <> nil then
			itsHorizSBar.SetValue(hPos div hUnit);

		if itsVertSBar <> nil then
			itsVertSBar.SetValue(vPos div vUnit);
	end;


{****************************************************}
{ ChangeSize (OVERRIDE)}
{}
{		Change the size of a ScrollPane}
{}
{****************************************************}

	procedure CScrollPane.ChangeSize (delta: Rect; redraw: Boolean);
	begin
		inherited ChangeSize(delta, redraw);
		AdjustScrollMax;
		Calibrate;
	end;



{ **** S C R O L L   P E R F O R M A N C E   M E T H O D S **** }



{****************************************************}
{ DoHorizScroll}
{}
{		Horizontally scroll the Panorama associated with a ScrollPane}
{}
{****************************************************}

	procedure CScrollPane.DoHorizScroll (whichPart: integer);
		var
			delta,						{ Number of pixels to scroll		}
			oldValue,					{ Current scroll bar setting		}
			minmax: integer;		{ Minimum or Maximum delta		}
			ticks: longint;			{ Tick count at end of Delay		}

	begin
		case whichPart of

			inUpButton: 
				delta := -hStep;

			inDownButton: 
				delta := hStep;

			inPageUp: 
				begin
					Delay(PAGE_DELAY, ticks);
					delta := hOverlap - hSpan;
				end;

			inPageDown: 
				begin
					Delay(PAGE_DELAY, ticks);
					delta := hSpan - hOverlap;
				end;
		end;

		oldValue := itsHorizSBar.GetValue;
		if delta < 0 then
			begin
				minmax := itsHorizSBar.GetMinValue - oldValue;
				if delta < minmax then
					delta := minmax;

			end
		else
			begin
				minmax := itsHorizSBar.GetMaxValue - oldValue;
				if delta > minmax then
					delta := minmax;
			end;

		if delta <> 0 then
			begin
				itsHorizSBar.SetValue(oldValue + delta);
				itsPanorama.Scroll(delta, 0, TRUE);
				itsHorizSBar.Prepare;
			end;
	end;


{****************************************************}
{ DoVertScroll}
{}
{		Vertically scroll the Panorama associated with a ScrollPane}
{}
{****************************************************}

	procedure CScrollPane.DoVertScroll (whichPart: integer);
		var
			delta,						{ Number of pixels to scroll		}
			oldValue,					{ Current scroll bar setting		}
			minmax: integer;		{ Minimum or Maximum delta		}
			ticks: longint;			{ Tick count at end of Delay		}

	begin
		case whichPart of

			inUpButton: 
				delta := -vStep;

			inDownButton: 
				delta := vStep;

			inPageUp: 
				begin
					Delay(PAGE_DELAY, ticks);
					delta := vOverlap - vSpan;
				end;

			inPageDown: 
				begin
					Delay(PAGE_DELAY, ticks);
					delta := vSpan - vOverlap;
				end;
		end;

		oldValue := itsVertSBar.GetValue;
		if delta < 0 then
			begin
				minmax := itsVertSBar.GetMinValue - oldValue;
				if delta < minmax then
					delta := minmax;
			end

		else
			begin
				minmax := itsVertSBar.GetMaxValue - oldValue;
				if delta > minmax then
					delta := minmax;
			end;

		if delta <> 0 then
			begin
				itsVertSBar.SetValue(oldValue + delta);
				itsPanorama.Scroll(0, delta, TRUE);
				itsVertSBar.Prepare;
			end;
	end;


{****************************************************}
{ DoThumbDrag}
{}
{		Adjust associated Panorama when the thumb of a scroll bar is dragged}
{}
{****************************************************}

	procedure CScrollPane.DoThumbDrag (hDelta, vDelta: integer);
	begin
		itsPanorama.Scroll(hDelta, vDelta, TRUE);
	end;

end.