{****************************************************}
{ 		CView.p}
{}
{		The View Class}
{}
{		Abstract class for objects which have a visual representation }
{		on the screen. }
{}
{		SUPERCLASS = CBureaucrat}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CView;

interface

	uses
		TCL;


implementation


{****************************************************}
{ IView}
{}
{		Initialize a View object. Views initially have no subviews, and}
{		are invisible and inactive.}
{}
{****************************************************}

	procedure CView.IView (anEnclosure: CView; aSupervisor: CBureaucrat);
	begin
		IBureaucrat(aSupervisor);			{ Initialize superclass			}
		macPort := nil;						{ Subclass should set the port	}
		itsEnclosure := anEnclosure;		{ Store enclosing view			}
		itsSubviews := nil;
		visible := FALSE;						{ By default, views are invisible,	}
		active := FALSE;						{   inactive, and						}
		wantsClicks := FALSE;				{   don't handle mouse clicks			}
	end;


{****************************************************}
{ IViewRes}
{}
{		Initialize a View from a resource}
{}
{****************************************************}

	procedure CView.IViewRes (rType: ResType; resID: integer; anEnclosure: CView; aSupervisor: CBureaucrat);
		var
			hRes: Handle;		{ Handle to resource template		}

	begin
		hRes := GetResource(rType, resID);	{ Load resource into memory	}
		CheckResource(hRes);					{ Make sure resource exists		}
		HLock(hRes);								{ Lock resource in heap			}

		IViewTemp(anEnclosure, aSupervisor, hRes^);	{ Initialize View from template	}

		HUnlock(hRes);							{ Unlock resource					}
	end;


{****************************************************}
{ IViewTemp}
{}
{		Initialize a View from a template}
{}
{****************************************************}

	procedure CView.IViewTemp (anEnclosure: CView; aSupervisor: CBureaucrat; viewData: Ptr);
		var
			p: ViewTempP;

	begin
		IBureaucrat(aSupervisor);		{ Initialize superclass				}
		macPort := nil;					{ Subclass should set the port		}
		itsEnclosure := anEnclosure;	{ Store enclosing view				}
		itsSubviews := nil;

		p := ViewTempP(viewData);		{ Set instance vars from template	}
		visible := p^.visible <> 0;
		active := p^.active <> 0;
		wantsClicks := p^.wantsClicks <> 0;
	end;


{****************************************************}
{ Free}
{}
{		Dispose of a view and all its subviews}
{}
{****************************************************}

	procedure CView.Free;
	begin
		if itsSubviews <> nil then		{ If this view has subviews ...		}
			itsSubviews.DisposeAll;		{   Throw them out					}
		if itsEnclosure <> nil then
			itsEnclosure.RemoveSubview(SELF);
		if SELF = gLastViewHit then
			gLastViewHit := nil;
		inherited Free;					{ Pass message on to superclass	}
	end;


{****************************************************}
{ Show}
{}
{		Make a view visible}
{}
{****************************************************}

	procedure CView.Show;
	begin
		visible := TRUE;
	end;


{****************************************************}
{ Hide}
{}
{		Make a view invisible}
{}
{****************************************************}

	procedure CView.Hide;
	begin
		visible := FALSE;
		if gGopher = SELF then
			gGopher := itsEnclosure;
	end;


{****************************************************}
{ Activate}
{}
{		Make a view active. Also activate all its subviews.}
{}
{****************************************************}

	procedure View_Activate (theView: CView);

				{Activate a View}

	begin
		if (theView.active) and (theView.visible) then
			begin
				theView.active := FALSE;
				theView.Activate;
			end;
	end;


	procedure CView.Activate;
	begin
		active := TRUE;						{ Set active flag					}
		if itsSubviews <> nil then
			itsSubviews.DoForEach(View_Activate);
	end;


{****************************************************}
{ Deactivate}
{}
{		Make a view inactive}
{}
{****************************************************}

	procedure View_Deactivate (theView: CView);

				{Deactivate a View}

		var
			saveActive: Boolean;

	begin
		if theView.visible then
			begin
				saveActive := theView.active;
				theView.Deactivate;
				theView.active := saveActive;
			end;
	end;


	procedure CView.Deactivate;
	begin
		active := FALSE;
		if itsSubviews <> nil then
			itsSubviews.DoForEach(View_Deactivate);
		if gGopher = SELF then
			gGopher := itsSupervisor;
	end;


{****************************************************}
{ IsVisible}
{}
{		Check whether a view is visible}
{}
{****************************************************}

	function CView.IsVisible: Boolean;
	begin
		IsVisible := visible;				{ Return instance variable			}
	end;


{****************************************************}
{ IsActive}
{}
{		Check whether a view is active}
{}
{****************************************************}

	function CView.IsActive: Boolean;
	begin
		if active then
			if itsEnclosure <> nil then
				IsActive := itsEnclosure.IsActive
			else
				IsActive := TRUE
		else
			IsActive := FALSE;
	end;


{****************************************************}
{	 ReallyVisible }
{}
{	        Check whether a view and all its enclosures are visible. }
{}
{****************************************************}

	function CView.ReallyVisible: Boolean;
	begin
		if visible then
			ReallyVisible := itsEnclosure.ReallyVisible
		else
			ReallyVisible := FALSE;
	end;


{****************************************************}
{	 DispatchClick }
{}
{	        Mouse was pressed somewhere in the view. Find out if a subview }
{	        which wants to handle mouse clicks was hit. If not, send }
{	        message to ourself to handle the click. }
{}
{	        NOTE: The where field of macEvent is in Window coordinates upon entry. }
{}
{****************************************************}

	procedure CView.DispatchClick (var macEvent: EventRecord);
		var
			hitView: CView;
			hitPt: Point;
			hOff, vOff: longint;

	begin
		hitView := FindSubview(macEvent.where);
		if hitView = nil then
			begin
				CountClicks(SELF, macEvent);
				Prepare;
				hitPt := macEvent.where;
				GetOrigin(hOff, vOff);
				hitPt.h := hitPt.h + hOff;
				hitPt.v := hitPt.v + vOff;
				DoClick(hitPt, macEvent.modifiers, macEvent.when);
			end
		else
			hitView.DispatchClick(macEvent);
	end;


{****************************************************}
{ DoClick}
{}
{		Respond to a mouse click within the view. Default action is}
{		to do nothing.}
{}
{****************************************************}

	procedure CView.DoClick (hitPt: Point; modifierKeys: integer; when: longint);
	begin
	end;										{ Null Method						}


{****************************************************}
{ DoMouseUp}
{}
{		Respond to the mouse button being released after a click within}
{		the view. Default action is to do nothing.}
{}
{****************************************************}

	procedure CView.DoMouseUp (macEvent: EventRecord);
	begin
	end;										{ Null Method						}


{****************************************************}
{	 TrackMouse }
{}
{	        Follow the mouse while it is being held down and repeatedly inform }
{	        a specified Task of the mouse location }
{}
{****************************************************}

	procedure CView.TrackMouse (theTask: CMouseTask; startPt: Point; var pinRect: Rect);
		var
			currPt, prevPt: Point;
			macEvent: EventRecord;

	begin
		Prepare;
		theTask.BeginTracking(startPt);

		prevPt := startPt;
		while StillDown do
			begin
				Prepare;
				GetMouse(currPt);
				PinInRect(pinRect, currPt);
				theTask.KeepTracking(currPt, prevPt, startPt);
				prevPt := currPt;
			end;

		Prepare;
		if OSEventAvail(mUpMask, macEvent) then
			begin
				currPt := macEvent.where;
				GlobalToLocal(currPt);
			end;

		PinInRect(pinRect, currPt);
		theTask.EndTracking(currPt, prevPt, startPt);
	end;


{****************************************************}
{	 DispatchCursor }
{}
{	        Mouse is inside view and the cursor shape needs to be adjusted. }
{	        Find out if a subview can set the cursor shape. If not, send }
{	        message to ourself to handle it. }
{}
{****************************************************}

	procedure View_SubFrame (theView: CView; uncovered: Ptr);

				{Find area of View not covered by subviews}

		type
			RgnHandlePtr = ^RgnHandle;

		var
			frame: Rect;

	begin
		if (theView.IsVisible) and (theView.wantsClicks) then
			begin
				theView.GetFrame(frame);
				theView.FrameToGlobalR(frame);
				RectRgn(gUtilRgn, frame);
				DiffRgn(RgnHandlePtr(uncovered)^, gUtilRgn, RgnHandlePtr(uncovered)^);
			end;
	end;


	procedure CView.DispatchCursor (where: Point; mouseRgn: RgnHandle);
		var
			hitView: CView;		{ View containing the mouse		}
			aperture: Rect;			{ Visible portion of view			}
			savePort: GrafPtr;
			saveOrigin: Point;
			offset: Point;
	begin
		hitView := FindSubview(where);		{ See if mouse is within a subview	}

		if hitView = nil then
			begin							{ Mouse not within a subview		}

										{ Calculate the uncovered area of	}
										{   this View.							}

				GetAperture(aperture);			{ Start off with the aperture	}
				FrameToGlobalR(aperture);		{   in global coordinates			}
				RectRgn(mouseRgn, aperture);

				if itsSubviews <> nil then		{ Subtract the area covered by	}
											{   the frames of its subviews	}

					itsSubviews.DoForEach1(View_SubFrame, @mouseRgn);

				{ Now take intersection with the visRgn of View's			}
				{ port. There could be something, such as a window,		}
				{ floating above the View. Unfortunately, the visRgn		}
				{ is in local coordinates and we need it to be in			}
				{ global coords. Rather than making a copy and			}
				{ offsetting to global coords, we temporily shift the		}
				{ View's port so that its local coords are the same as		}
				{ global coords. This saves us the overhead of making	}
				{ a copy of the visRgn, which may take up a lot of			}
				{ memory, at the expense of having to save and restore	}
				{ the port and origin.											}

				GetPort(savePort);				{ Save port and origin			}
				SetPort(macPort);
				saveOrigin := topLeft(macPort^.portRect);
				offset := saveOrigin;
				LocalToGlobal(offset);
				SetOrigin(offset.h, offset.v);
				  							{ visRgn is in Global coords now	}

				SectRgn(macPort^.visRgn, mouseRgn, mouseRgn);

											{ Restore origin and port			}

				SetOrigin(saveOrigin.h, saveOrigin.v);
				SetPort(savePort);

				AdjustCursor(where, mouseRgn);	{ Set cursor shape and possibly		}
												{   further restrict the mouseRgn	}

			end
		else 							{ Mouse is within a subview		}

			hitView.DispatchCursor(where, mouseRgn);
	end;



{****************************************************}
{ AdjustCursor}
{}
{		Adjust the shape of the cursor according the position of the mouse.}
{		Also, determine the region in which the mouse shape remains the same.}
{}
{****************************************************}

	procedure CView.AdjustCursor (where: Point; mouseRgn: RgnHandle);
	begin
		SetCursor(arrow);		{ Default action is to let the		}
	end;							{   enclosure do the work		}


{****************************************************}
{ GetMacPort}
{}
{		Return a pointer to the Mac GrafPort of the View}
{}
{****************************************************}

	function CView.GetMacPort: GrafPtr;
	begin
		GetMacPort := macPort;			{ Return instance variable			}
	end;


{****************************************************}
{ GetOrigin}
{}
{		Return the origin of the view}
{}
{****************************************************}

	procedure CView.GetOrigin (var theHOrigin, theVOrigin: longint);
	begin
		theHOrigin := 0;
		theVOrigin := 0;
	end;


{****************************************************}
{ GetFrame}
{}
{		Return the frame of the view.  Subclasses must override. }
{}
{****************************************************}

	procedure CView.GetFrame (var theFrame: Rect);
	begin
	end;


{****************************************************}
{ GetInterior}
{}
{		Return the interior of the view. This method assumes that the}
{		interior is the same as the frame. Subclasses must override this}
{		method if this is not the case.}
{}
{****************************************************}

	procedure CView.GetInterior (var theInterior: Rect);
	begin
		GetFrame(theInterior);
	end;


{****************************************************}
{	 GetAperture }
{}
{	        Return the portion of the view which is actually visible. }
{	        Subclasses must override this method. }
{}
{****************************************************}

	procedure CView.GetAperture (var theAperture: Rect);
	begin
	end;


{****************************************************}
{ AddSubview}
{}
{	        Add a subview. Check to make sure a subview list exists, and create }
{	        one if it doesn't. This saves a little space because a subview list }
{	        isn't created until the first subview is added. Views without }
{	        subviews don't have to have a reference to an empty List object. }
{}
{****************************************************}

	procedure CView.AddSubview (theSubview: CView);
        var
            theViewList: CList;     { Altered by TCL Weaver 1.0 (5/9/90) }

	begin
		if itsSubviews = nil then
			begin
                new(theViewList);       { Altered by TCL Weaver 1.0 (5/9/90) }
                itsSubviews := theViewList;
				itsSubviews.IList;
			end;
		itsSubviews.Add(theSubview);
	end;


{****************************************************}
{	 RemoveSubview }
{}
{	        Remove a subview }
{}
{****************************************************}

	procedure CView.RemoveSubview (theSubview: CView);
	begin
		itsSubviews.Remove(theSubview);
	end;


{****************************************************}
{	 FindSubview }
{}
{	        Find which subview is hit by a given point and wants clicks. Point }
{	        is specified in window coordinates. }
{}
{****************************************************}

	function View_Contains (theView: CView; thePoint: Ptr): Boolean;
	begin
		if theView.visible then
			View_Contains := theView.Contains(PointPtr(thePoint)^)
		else
			View_Contains := FALSE;
	end;


	function CView.FindSubview (hitPt: Point): CView;
		var
			hitSubview: CView;
			temp: CObject;

	begin
		if itsSubviews = nil then
			FindSubview := nil
		else
			begin
				temp := itsSubviews.LastSuccess1(View_Contains, @hitPt);
				if temp = nil then
					FindSubview := nil
				else
					begin
						hitSubview := CView(temp);
						if not hitSubview.wantsClicks then
							FindSubview := nil
						else
							FindSubview := hitSubview;
					end;
			end;
	end;


{****************************************************}
{	 Contains }
{}
{	        Check whether a view contains a specified point. Subclasses must }
{	        override this method. }
{}
{****************************************************}

	function CView.Contains (thePoint: Point): Boolean;
	begin
	end;


{****************************************************}
{	 HitSamePart }
{}
{	        Check whether two points hit the same part of the view. The two }
{	        points must actually be within the view (return TRUE from the }
{	        Contain method). The test is used for checking the "close in location" }
{	        requirement for multiple clicks. This means that the same "part" of the }
{	        view is hit by the two points. For text views, a "part" could be a word; }
{	        for graphical views, a "part" could be a graphical object. This }
{	        method assumes that a view has only one part, so all points hit the }
{	        same part. Subclasses which support multiple clicks and have more }
{	        than one part must override this method. }
{}
{****************************************************}

	function CView.HitSamePart (var pointA, pointB: Point): Boolean;
	begin
		HitSamePart := TRUE;
	end;


{****************************************************}
{	 SubpaneLocation }
{}
{	        Determine the Window coords corresponding to a point which specifies }
{	        the location of a subview. This method assumes that the View uses }
{	        pixel values to specify subview locations. Subclasses which use a }
{	        unit different from one pixel must override this method. }
{}
{****************************************************}

	procedure CView.SubpaneLocation (hEncl, vEncl: integer; var hLocation, vLocation: longint);
		var
			theHOrigin, theVOrigin: longint;

	begin
		GetOrigin(theHOrigin, theVOrigin);
		hLocation := hEncl - theHOrigin;
		vLocation := vEncl - theVOrigin;
	end;


{****************************************************}
{	 Prepare }
{}
{	        Do whatever preparations are necessary for drawing in a view. }
{	        Subclasses must override this method. }
{}
{****************************************************}

	procedure CView.Prepare;
	begin
	end;


{****************************************************}
{ FrameToGlobalR}
{}
{		Convert a rectangle from Frame to Global coordinates}
{}
{****************************************************}

	procedure CView.FrameToGlobalR (var theRect: Rect);
	begin
	end;


{****************************************************}
{ SetWantsClicks}
{}
{		Specify whether or not the View responds to mouse clicks}
{}
{****************************************************}

	procedure CView.SetWantsClicks (aWantsClicks: Boolean);
	begin
		wantsClicks := aWantsClicks;
	end;


end.