{****************************************************}
{ 		CWindow.p}
{}
{		The Window Class}
{}
{		A standard Mac window. Normally, all drawing within a window is }
{		performed by installing Panes in the window. This class implements }
{		the standard Mac behavior of overlapping windows which may be }
{		dragged, resized, zoomed, closed, etc. }
{}
{		SUPERCLASS = CView}
{}
{		Copyright  1989, Symantec Corporation.  All rights reserved.			}
{}
{****************************************************}

unit CWindow;

interface

	uses
		TCL;


implementation


{*	Template for WIND resource	*}

	type
		WINDtemplate = record
				boundsRect: Rect;
				WDEFid, visible, goAwayFlag: integer;
				refCon: longint;
				title: Str255;
			end;

		WINDtemplateP = ^WINDtemplate;
		WINDtemplateH = ^WINDtemplateP;



{ **** C O N S T R U C T I O N / D E S T R U C T I O N     M E T H O D S **** }



{****************************************************}
{ IWindow}
{}
{		Initialize a Window object}
{}
{****************************************************}

	procedure CWindow.IWindow (WINDid: integer; aFloating: Boolean; anEnclosure: CDesktop; aSupervisor: CDirector);
		var
			theWIND: WINDtemplateH;
			theWINDP: WINDtemplateP;
			saveVis: integer;
			behindWindow: WindowPtr;

	begin

		IView(anEnclosure, aSupervisor);		{ Initialize superclass.	}

{	         Because we maintain our own window list and have to do	}
{	         a lot of manual activating/deactivating to support       	}
{	         floating windows, it is imperative that windows be      	}
{	         invisible when initially created. Therefore, we read the 	}
{	         the WIND resource template and set the visible flag to   	}
{	         FALSE (after saving the original value).                 		}

		theWIND := WINDtemplateH(GetResource('WIND', WINDid));
		CheckResource(Handle(theWIND));
		theWINDP := theWIND^;
		saveVis := theWINDP^.visible;
		theWINDP^.visible := 0;

		floating := aFloating;

		MakeMacWindow(WINDid);
		SetWRefCon(macPort, longint(SELF));

		WindowPeek(macPort)^.windowKind := OBJ_WINDOW_KIND;

		sizeRect := GetGrayRgn^^.rgnBBox;
		sizeRect.left := MIN_WSIZE;
		sizeRect.top := MIN_WSIZE;

		actClick := FALSE;
		wantsClicks := TRUE;

		CDesktop(itsEnclosure).AddWind(SELF);

		if saveVis <> 0 then
			begin
				theWIND := WINDtemplateH(GetResource('WIND', WINDid));
				theWIND^^.visible := 1;
				Select;
			end;
	end;


{****************************************************}
{ Free (OVERRIDE)}
{}
{		Get rid of a window and all its panes}
{}
{****************************************************}

	procedure CWindow.Free;
	begin
		if visible then					{ Hide window first so the user	}
			Hide;								{   doesn't see the ugly details	}
										{   as each subview is disposed	}

		if itsSubviews <> nil then		{ If this window has subviews ...	}
			itsSubviews.DisposeAll;		{   Throw them out					}
		itsSubviews := nil;				{ Clear dangling handle				}

										{ Remove window from desktop		}

		CDesktop(itsEnclosure).RemoveWind(SELF);
		itsEnclosure := nil;

		DisposeWindow(macPort);		{ Kill the Toolbox window stuff		}

		inherited Free;					{ Pass message on to superclass	}
	end;


{****************************************************}
{	 MakeMacWindow }
{}
{	        Create a new Toolbox window record using the specified WIND }
{	        resource ID. This method creates a normal Mac window and lets }
{	        the window manager allocate space for the window record. Subclasses }
{	        should override this method to perform their own memory allocation }
{	        or to create a color window. For compatibility with the Desktop }
{	        class, the convention of putting floating windows in front and }
{	        non-floating windows in back should be observed. }
{}
{****************************************************}

	procedure CWindow.MakeMacWindow (WINDid: integer);
		var
			bottomWindow: CWindow;

	begin
		if floating then
			macPort := GetNewWindow(WINDid, nil, WindowPtr(-1))
		else
			begin		{ Put the window behind the application's bottom window, but 	}
					{ in front of THINK Pascal's windows (if any). 						}

				bottomWindow := gDesktop.GetBottomWindow;
				if bottomWindow = nil then
					macPort := GetNewWindow(WINDid, nil, WindowPtr(-1))
				else
					macPort := GetNewWindow(WINDid, nil, bottomWindow.GetMacPort);
			end;
	end;


{****************************************************}
{ Close}
{}
{		Respond to a click in the close box}
{}
{****************************************************}

	procedure CWindow.Close;
	begin
		CDirector(itsSupervisor).CloseWind(SELF);
	end;



{ **** A C C E S S I N G   M E T H O D S **** }



{****************************************************}
{	 GetFrame (OVERRIDE)}
{}
{	        Return the frame of the window }
{}
{****************************************************}

	procedure CWindow.GetFrame (var theFrame: Rect);
	begin
		theFrame.left := -1;		{ Frame of window includes a one pixel }
		theFrame.top := -1;		{ border around its port rectangle 		}

		theFrame.right := macPort^.portRect.right - macPort^.portRect.left + 1;
		theFrame.bottom := macPort^.portRect.bottom - macPort^.portRect.top + 1;
	end;


{****************************************************}
{ GetInterior (OVERRIDE)}
{}
{		Return the interior of the window}
{}
{****************************************************}

	procedure CWindow.GetInterior (var theInterior: Rect);
	begin
								{ Window interior is the same	}
								{   size as its port rectangle		}
		theInterior.left := 0;
		theInterior.top := 0;
		theInterior.right := macPort^.portRect.right - macPort^.portRect.left;
		theInterior.bottom := macPort^.portRect.bottom - macPort^.portRect.top;
	end;


{****************************************************}
{	 GetAperture {OVERRIDE) }
{}
{	        Return the area of the window where drawing may occur. For a }
{	        window, this is the entire port rectangle. The top left of the }
{	        window is always at (0, 0), so the right side is the width and }
{	        the bottom side is the height of the port rectangle. }
{}
{****************************************************}

	procedure CWindow.GetAperture (var theAperture: Rect);
	begin
		theAperture.left := 0;
		theAperture.top := 0;
		theAperture.right := macPort^.portRect.right - macPort^.portRect.left;
		theAperture.bottom := macPort^.portRect.bottom - macPort^.portRect.top;
	end;


{****************************************************}
{ IsFloating}
{}
{		Check if a window is a floating window}
{}
{****************************************************}

	function CWindow.IsFloating: Boolean;
	begin
		IsFloating := floating;			{ Return instance variable			}
	end;


{****************************************************}
{ SetTitle}
{}
{		Specify a title for a window}
{}
{****************************************************}

	procedure CWindow.SetTitle (theTitle: Str255);
	begin
		SetWTitle(macPort, theTitle);
	end;


{****************************************************}
{ GetTitle}
{}
{		Return the title of a window}
{}
{****************************************************}

	procedure CWindow.GetTitle (var theTitle: Str255);
	begin
		GetWTitle(macPort, theTitle);
	end;


{****************************************************}
{ SetActClick}
{}
{		Specify the whether the window processes the mouse click which}
{		activates the window}
{}
{****************************************************}

	procedure CWindow.SetActClick (anActClick: Boolean);
	begin
		actClick := anActClick;
	end;


{****************************************************}
{ WantsActClick}
{}
{		Check if a window processes a click which activates the window}
{}
{****************************************************}

	function CWindow.WantsActClick: Boolean;
	begin
		WantsActClick := actClick;			{ Return instance variable			}
	end;


{****************************************************}
{	 Contains {OVERRIDE) }
{}
{	        Check whether a Window contains a specified point. Point is }
{	        specified in global coordinates. Window contains a point if it }
{	        is inside the window's content region. }
{}
{****************************************************}

	function CWindow.Contains (thePoint: Point): Boolean;
	begin
		Contains := PtInRgn(thePoint, WindowPeek(macPort)^.contRgn);
	end;


{****************************************************}
{ SetSizeRect}
{}
{		Set the rectangle which specifies the maximum and minimum dimensions}
{		of a Window. The left and top of this rectangle are the minimum}
{		width and height; the right and bottom are the maximums.}
{}
{		This method also adjusts the standard zoom out state if necessary,}
{		so that zooming does not grow the window larger than the maximum size.}
{}
{****************************************************}

	procedure CWindow.SetSizeRect (aSizeRect: Rect);
		var
			zoomData: WStateDataH;
			theStdState: Rect;

	begin

		sizeRect := aSizeRect;			{ Set instance variable			}

										{ Check if window supports zooming	}

		if WindowPeek(macPort)^.spareFlag then
			begin
										{ Get standard state rectangle	}

				zoomData := WStateDataH(WindowPeek(macPort)^.dataHandle);
				theStdState := zoomData^^.stdState;

										{ Adjust width					}

				if aSizeRect.right < (theStdState.right - theStdState.left) then
					theStdState.right := theStdState.left + aSizeRect.right;

										{ Adjust height					}

				if aSizeRect.bottom < (theStdState.bottom - theStdState.top) then
					theStdState.bottom := theStdState.top + aSizeRect.bottom;

				zoomData^^.stdState := theStdState;
			end;
	end;


{****************************************************}
{ SetStdState}
{}
{		Set the size and location for the window when it is zoomed out.}
{		Calling routine should make sure that the window supports zooming}
{		by checking the spareFlag field of its Mac window record.}
{}
{****************************************************}

	procedure CWindow.SetStdState (aStdState: Rect);
		var
			zoomData: WStateDataH;

	begin
		zoomData := WStateDataH(WindowPeek(macPort)^.dataHandle);
		zoomData^^.stdState := aStdState;
	end;



{ **** A P P E A R A N C E    M E T H O D S **** }



{****************************************************}
{ Show (OVERRIDE)}
{}
{		Make a window visible}
{}
{****************************************************}

	procedure CWindow.Show;
	begin
		if not visible then
			begin
				inherited Show;
				CDesktop(itsEnclosure).ShowWind(SELF);
			end;
	end;


{****************************************************}
{ Hide (OVERRIDE)}
{}
{		Make a window invisible}
{}
{****************************************************}

	procedure CWindow.Hide;
	begin
		if visible then
			begin
				inherited Hide;
				CDesktop(itsEnclosure).HideWind(SELF);
			end;
	end;


{****************************************************}
{ Activate (OVERRIDE)}
{}
{		Make a window active}
{}
{****************************************************}

	procedure CWindow.Activate;
	begin
		if not active then
			begin
				HiliteWindow(macPort, TRUE);
				inherited Activate;
				CDirector(itsSupervisor).ActivateWind(SELF);
			end;
	end;


{****************************************************}
{ Deactivate (OVERRIDE)}
{}
{		Make a window inactive}
{}
{****************************************************}

	procedure CWindow.Deactivate;
	begin
		if active then
			begin
				HiliteWindow(macPort, FALSE);
				inherited Deactivate;
				CDirector(itsSupervisor).DeactivateWind(SELF);
			end;
	end;


{****************************************************}
{ Select}
{}
{		Select a window by bringing it to the front and making it active}
{}
{****************************************************}

	procedure CWindow.Select;
	begin
		CDesktop(itsEnclosure).SelectWind(SELF);		{ Tell enclosure to select us		}
		visible := TRUE;
	end;


{****************************************************}
{ ShowResume}
{}
{		Make a window visible when the application resumes}
{}
{****************************************************}

	procedure CWindow.ShowResume;
	begin
		if not visible then
			begin
				inherited Show;
				MoveWindow(macPort, location.h, location.v, FALSE);
			end;
	end;


{****************************************************}
{ HideSuspend}
{}
{	        Make a window invisible when the application suspends. This method }
{	        "hides" the window by moving it offscreen. The original }
{	        location is saved so we can put the window back when the }
{	        application resumes. }
{}
{****************************************************}

	procedure CWindow.HideSuspend;
	begin
		if visible then
			begin
				inherited Hide;

				SetPort(macPort);
				location := topLeft(macPort^.portRect);
				LocalToGlobal(location);

				MoveOffScreen;
			end;
	end;


{****************************************************}
{ ShowOrHide (OVERRIDE)}
{}
{		Make a window visible or invisible , without performing (or generating) }
{		any activates or deactivates . }
{}
{		showFlag = TRUE, show window }
{		showFlag = FALSE , hide window }
{}
{****************************************************}

	procedure CWindow.ShowOrHide (showFlag: Boolean);
	begin
		ShowHide(macPort, showFlag);
		visible := showFlag;
	end;



{ **** S I Z E   A N D   L O C A T I O N   M E T H O D S **** }



{****************************************************}
{ Drag}
{}
{		Drag a window on the desktop}
{}
{****************************************************}

	procedure CWindow.Drag (macEvent: EventRecord);
	begin
		CDesktop(itsEnclosure).DragWind(SELF, macEvent);
	end;


{****************************************************}
{ Resize}
{}
{		Dynamically resize a window}
{}
{****************************************************}

	procedure CWindow.Resize (macEvent: EventRecord);
		var
			newSize,				{ New width and height for window	}
			oldSize: longint;	{ Current width and height				}
			theRect: Rect;		{ Utility rectangle						}

	begin
		theRect := macPort^.portRect;	{ Record the current width and	}
											{   height of the window in a long	}
											{   with the height in the hi word	}
											{   and the width in the lo word	}

		oldSize := BSL(theRect.bottom - theRect.top, 16) + (theRect.right - theRect.left);

		theRect := sizeRect;					{ Store instance variable in local	}
											{   var in case memory moves	}

		theRect.right := theRect.right + 1;		{ GrowWindow trap is off by one		}
		theRect.bottom := theRect.bottom + 1;	{	when growing window to its max	}
													{	size.  Most likely caused by		}
													{	using the PinRect trap.				}

			  	  						{ Let Toolbox draw the grow image	}
			  	  						{   newSize has the new height in		}
			  	  						{   the hi word, width in lo word		}

		newSize := GrowWindow(macPort, macEvent.where, theRect);

		{ Check whether the size changed. Inside Mac says that a newSize	}
		{ of zero means that the size is the same. However, it does not		}
		{ say that newSize will be zero in all cases where the size			}
		{ remains the same. Let's perform our own check of whether the	}
		{ size has changed. 															}

		if (newSize <> 0) and (newSize <> oldSize) then

										{ Set window to the new size		}

			ChangeSize(LoWord(newSize), HiWord(newSize));
	end;


{****************************************************}
{ Zoom}
{}
{		Zoom a window out to a standard size or in to its original size}
{}
{****************************************************}

	procedure CWindow.Zoom (direction: integer);
		var
			delta: Rect;			{ Change in size of window			}

	begin
		Prepare;
		delta := macPort^.portRect;		{ Save current window size			}

		ClipRect(delta);						{ White out contents of window		}
		EraseRect(delta);
										{ Toolbox call performs the zoom	}

		ZoomWindow(macPort, direction, FALSE);

		if itsSubviews <> nil then
			begin							{ Adjust size of all subviews		}
										{ Find change in window size			}

				delta.right := macPort^.portRect.right - delta.right;
				delta.bottom := macPort^.portRect.bottom - delta.bottom;
				itsSubviews.DoForEach1(Pane_AdjustToEnclosure, @delta);

				InvalRect(macPort^.portRect);		{ Force redraw of entire window	}
			end;

		if floating then
			CDesktop(itsEnclosure).SelectWind(SELF);
	end;


{****************************************************}
{ Move}
{}
{		Move a window to a new location by specifying the global coordinates}
{		at which to place the top left corner of the port rectangle of the}
{		window. Note that the window's title bar, if present, will lie above}
{		the specified vertical coordinate.}
{}
{****************************************************}

	procedure CWindow.Move (hGlobal, vGlobal: integer);
	begin
										{ Toolbox call does all the work.	}
										{   The final FALSE parameter		}
										{   means that we don't want to	}
										{   bring the window to the front	}
										{   if it is an underlying window.	}

		MoveWindow(macPort, hGlobal, vGlobal, FALSE);
	end;


{****************************************************}
{ ChangeSize}
{}
{		Change the size of a window by specifying the width and height}
{		in pixels. }
{}
{****************************************************}

	procedure CWindow.ChangeSize (width, height: integer);
		var
			theRect: Rect;

	begin
		theRect.left := 0;
		theRect.top := 0;
		theRect.right := macPort^.portRect.right - macPort^.portRect.left;
		theRect.bottom := macPort^.portRect.bottom - macPort^.portRect.top;

		width := Min(width, sizeRect.right);
		height := Min(height, sizeRect.bottom);
		width := Max(width, sizeRect.left);
		height := Max(height, sizeRect.top);

										{ Toolbox call does all the work.		}
										{   The final TRUE parameter means	}
										{   that we want any newly exposed	}
										{   area to be added to the update		}
										{   region of the window.				}

		SizeWindow(macPort, width, height, TRUE);

		if itsSubviews <> nil then
			begin
				theRect.right := width - theRect.right;
				theRect.bottom := height - theRect.bottom;
				itsSubviews.DoForEach1(Pane_AdjustToEnclosure, @theRect);
			end;

		Update;		{ Update the window }
	end;


{****************************************************}
{	 MoveOffScreen }
{}
{	        Move a window offscreen by placing the left edge of the window past }
{	        the right side of the desktop. }
{}
{****************************************************}

	procedure CWindow.MoveOffScreen;
		var
			deskBounds: Rect;

		{ ??? Stick the top left of the port rectangle 1000 pixels to			}
		{ the right and below the bottom right corner of the desktop.			}
		{ This could cause a problem for windows with a structure			}
		{ region larger than 1000 pixels than the content region or			}
		{ for extremely large windows (on the order of 32000 pixels)		}
		{ where moving the window may cause arithmetic overflow. Such	}
		{ situations are very rare -- you really have to go out of			}
		{ your way to come up with a practical reason for having			}
		{ windows which are so large.											}

	begin
		CDesktop(itsEnclosure).GetBounds(deskBounds);
		MoveWindow(macPort, deskBounds.right + 1000, deskBounds.bottom + 1000, FALSE);
	end;



{ **** D R A W I N G    M E T H O D S **** }



{****************************************************}
{ Update}
{}
{		Update the contents of a window}
{}
{****************************************************}

	procedure CWindow.Update;
		var
			savePort: GrafPtr;				{ The current port					}
			updateRect: Rect;				{ Bounding box of update region		}

	begin
		GetPort(savePort);				{ Save the original port			}
		Prepare;
		BeginUpdate(macPort);			{ Start the update process		}
										{ This restricts the visible area	}
										{   to just the update region,		}
										{   meaning that no drawing will	}
										{   occur outside this region		}

		ClipRect(macPort^.portRect);		{ Clip to the entire window		}
		EraseRect(macPort^.portRect);		{ Erase the update region		}

		if itsSubviews <> nil then
			begin    						{ Draw all subviews				}

				updateRect := macPort^.visRgn^^.rgnBBox;
				itsSubviews.DoForEach1(Pane_Draw, @updateRect);
			end;

		EndUpdate(macPort);				{ End the update process			}
		SetPort(savePort);				{ Restore the original port		}
	end;


{****************************************************}
{	 Prepare {OVERRIDE) }
{}
{	        Set up the Window's port and coordinate system }
{}
{****************************************************}

	procedure CWindow.Prepare;
	begin
		SetPort(macPort);
		SetOrigin(0, 0);
	end;



{ **** M O U S E   M E T H O D S **** }



{****************************************************}
{	 DispatchClick {OVERRIDE) }
{}
{	        Mouse click occurred inside a Window. Convert the mouse location }
{	        in the mouse down event record from Global to Window coordinates. }
{	        Then, let the superclass handle things. }
{}
{****************************************************}

	procedure CWindow.DispatchClick (var macEvent: EventRecord);
	begin
		Prepare;
		GlobalToLocal(macEvent.where);
		inherited DispatchClick(macEvent);
	end;


{****************************************************}
{ DispatchCursor (OVERRIDE)}
{}
{		Find subview which wants to set the shape of the cursor. Point}
{		is passed in Global coordinates. Convert it to Window coordinates,}
{		then let superclass handle things.}
{}
{****************************************************}

	procedure CWindow.DispatchCursor (where: Point; mouseRgn: RgnHandle);
		var
			wTopLeft: Point;

	begin
										{ Convert point to Window coords	}

		wTopLeft := topLeft(WindowPeek(macPort)^.contRgn^^.rgnBBox);
		where.h := where.h - wTopLeft.h;
		where.v := where.v - wTopLeft.v;

										{ Let superclass take over			}

		inherited DispatchCursor(where, mouseRgn);
	end;


{****************************************************}
{ FrameToGlobalR (OVERRIDE)}
{}
{		Convert a rectangle from Frame to Global coordinates}
{}
{****************************************************}

	procedure CWindow.FrameToGlobalR (var theRect: Rect);
		var
			theOffset: Point;

	begin
    { Altered by TCL Weaver 1.0 (4/27/90) }

        theOffset := topLeft(WindowPeek(macPort)^.contRgn^^.rgnBBox);

        OffsetRect(theRect, theOffset.h, theOffset.v);
	end;


end.